import base64
import json

from django.core.mail.backends.base import BaseEmailBackend
from googleapiclient.http import HttpError

from MySU.services.google_mail import GoogleMail


class GmailBackend(GoogleMail, BaseEmailBackend):
    def __init__(self, fail_silently=False, **kwargs):
        BaseEmailBackend.__init__(self, fail_silently=fail_silently, **kwargs)
        GoogleMail.__init__(self)

    def construct_raw_message(self, email):
        """
        Converts the email message into the format required for the Gmail API.

         Args:
           # user_id: User's email address. The special value "me"
           # can be used to indicate the authenticated user.
           email: Message to be sent.

         Returns:
           Sent Message.
         """
        # Don't try to send a message if there are no receivers for the message
        if not email.recipients():
            return False
        raw_message = {'raw': base64.urlsafe_b64encode(email.message().as_bytes()).decode()}
        return self.api.users().messages().send(userId="me", body=raw_message)

    def send_messages(self, email_messages):
        """
        Send all messages using the Gmail API

        """
        if not email_messages:
            return 0
        msg_count = 0
        last_exception = None

        for message in email_messages:
            raw_message = self.construct_raw_message(message)
            try:
                raw_message.execute()
            except HttpError as error:
                data = json.loads(error.content.decode('utf-8'))
                message = data['error']['message']
                self.log.exception('An error occurred sending the message via GMail API:  %s', message)
                last_exception = error
            finally:
                msg_count += 1

        # Callback is commented, since it is only used in the now commented batch below
        # def send_callback(r_id, response, exception):
        #     nonlocal msg_count, last_exception
        #     if exception is not None:
        #         self.log.exception('An error occurred sending the message via GMail API:  %s', exception)
        #         last_exception = exception
        #     else:
        #         msg_count += 1

        # TODO When using batches we get the weird result that Media Requests are not supported.
        # batch = self.api.new_batch_http_request(send_callback)
        # for message in email_messages:
        #     batch.add(self.construct_raw_message(message))
        # batch.execute()
        if not self.fail_silently and last_exception:
            raise last_exception
        return msg_count
