import datetime
import json

from django.db.models import F
from django_cron import CronJobBase
from django_cron import Schedule

from apps.MySU.models import AssociationMembership
from MySU.objects import YearlessDate


class MembershipPeriodEndCron(CronJobBase):
    """
    This Cron handles what should happen to a membership once it period has ended. This can either of following things:
     - nothing
     - update the type of the membership to the new type the member wishes to have after the period
     - end the membership if the membership when
        - membership type auto ends
        - membership date left has been set
        - membership type end date reached
    """
    RUN_AT_TIMES = ['0:05']

    schedule = Schedule(run_at_times=RUN_AT_TIMES)
    code = "membership_period_end_cron"  # a unique code

    def do(self):
        today = datetime.date.today()
        yesterday = today - datetime.timedelta(1)
        yearless_yesterday = YearlessDate.from_date(yesterday)
        # Update status of membership that have ended
        membership_ended_data_left_count = AssociationMembership.objects\
            .current() \
            .filter(date_left=yesterday) \
            .update(status='Ended')
        # End all memberships that are set to auto end
        membership_ended_auto_end_count = AssociationMembership.objects\
            .current()\
            .filter(type__full_year_membership=True, association__full_year_membership_start_date=yearless_yesterday) \
            .filter(change_type_on_period_end=False) \
            .filter(type__auto_end=True)\
            .update(status='Ended', date_left=yesterday)
        # End all memberships with a type with a fixed end date
        membership_ended_type_end_date_count = AssociationMembership.objects \
            .current() \
            .filter(type__full_year_membership=False, type__end_date=yearless_yesterday) \
            .filter(change_type_on_period_end=False) \
            .update(status='Ended', date_left=yesterday)
        # update all those that have indicated a new_type
        membership_updated_type_end_date_count = AssociationMembership.objects\
            .current() \
            .filter(type__full_year_membership=False, type__end_date=yearless_yesterday) \
            .filter(change_type_on_period_end=True) \
            .update(type=F('new_type'), change_type_on_period_end=False, new_type=None)
        membership_updated_full_year_end_count = AssociationMembership.objects\
            .current()\
            .filter(type__full_year_membership=True, association__full_year_membership_start_date=yearless_yesterday) \
            .filter(change_type_on_period_end=True) \
            .update(type=F('new_type'), change_type_on_period_end=False, new_type=None)
        return json.dumps({
            "Memberships ended because of date_left": membership_ended_data_left_count,
            "Memberships ended because of auto_end": membership_ended_auto_end_count,
            "Memberships end because of type end_date": membership_ended_type_end_date_count,
            "Memberships updated because of type end_date": membership_updated_type_end_date_count,
            "Memberships updated because of full_year ended": membership_updated_full_year_end_count
        }, indent=2)
