import datetime
import re

from django.core.validators import ValidationError
from django.db.models import Field

from MySU.forms import YearlessDateForm
from MySU.objects import YearlessDate


class YearlessDateField(Field):
    """
    A model field for storing dates without years.Django only provides date and datetime fields.
    Sometimes we do not wish to store the year however, so this is a custom field that does so.
    """

    description = "A date without a year, for use in things like birthdays"
    default_error_messages = {
        'invalid': "'%(value)s' value has an invalid date format. It must be in MM-DD format.",
        'invalid_date': "'%(value)s' value has the correct format (MM-DD) but it is an invalid date.",
    }
    date_regex = re.compile(r'(?P<month>\d{1,2})-(?P<day>\d{1,2})$')

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 5
        super().__init__(*args, **kwargs)

    def parse_date(self, value):
        match = self.date_regex.match(value)
        if match:
            kw = {k: int(v) for k, v in match.groupdict().items()}
            return YearlessDate(**kw)

    def to_python(self, value):
        """
        Convert the input value into the expected Python data type, raising
        django.core.exceptions.ValidationError if the data can't be converted.
        Return the converted value.
        """
        if isinstance(value, YearlessDate):
            return value
        if isinstance(value, datetime.datetime) or isinstance(value, datetime.date):
            return YearlessDate(value.month, value.day)
        if value is None:
            return value
        # The string case.
        try:
            parsed_date = self.parse_date(value)
            if parsed_date is not None:
                return parsed_date
        except ValueError:
            raise ValidationError(
                self.error_messages['invalid_date'],
                code='invalid_date',
                params={'value': value},
            )

        raise ValidationError(
            self.error_messages['invalid'],
            code='invalid',
            params={'value': value},
        )

    def from_db_value(self, value, expression, connection):
        """Called during loading from database"""
        return self.to_python(value)

    def get_prep_value(self, value):
        """The reverse of to_python, for inserting into the database"""
        if value is None:
            return None
        return str(value)

    def get_internal_type(self):
        return 'CharField'

    def formfield(self, **kwargs):
        """
        This is a fairly standard way to set up some defaults
        while letting the caller override them.
        """
        return super().formfield(**{
            'form_class': YearlessDateForm,
            **kwargs,
        })
