import io

import pygraphviz as pgv
from django.core import management
from django.core.management.base import BaseCommand
from django.core.management.base import CommandError


class Command(BaseCommand):
    """
    Creates a .png of the relevant models in the project.

    The bulk of the work is done by the `graph_models` command from django_extensions. After being given a set of
    options that command can produce a UML diagram for the project. The layouts it supports are however insufficient.
    The automatically computed positions based on the layout parameter make a cluttered diagram. In this command we
    manually pass position data. How to do this is taken from the `miles_graph` example from the pygraphviz repository.

    The node_position_dict provides a mapping from the node name to a position. The dict is created running
    `
    with io.StringIO() as out:
        management.call_command("graph_models", all_applications=True, stdout=out)
        graph_dot_file = out.getvalue()
    graph = pgv.AGraph(graph_dot_file)
    node_position_dict = {node.name: "0,0)" for node in graph.nodes()}
    `
    This makes sure that the names of the nodes in the dict are consistent with the names of the nodes in the graph.
    Afterwards the nodes that are excluded from the final UML are commented, so it is clear that they are not in the
    graph, and not deleted, so the names of the nodes do not have to be recomputed when we decide to add them. The
    positions are manually set to make sure that things that are semantically close are close in the graph, e.g.
    event and enrollments are close and specific data and specific data fields are close, and a minimal number of lines
    cross each other.
    """
    help = 'Creates a UML diagram for the project'
    node_position_dict = {
        'MySU_models_association_Association': '0,0)',
        'MySU_models_association_membership_AssociationMembership': '800,600)',
        'MySU_models_association_specific_data_AssociationSpecificData': '800,200)',
        'MySU_models_association_specific_data_fields_AssociationSpecificDataFields': '800,0)',
        'MySU_models_debt_collection_DebtCollection': '-500,700)',
        'MySU_models_debt_entry_DebtEntry': '-500,1000)',
        'MySU_models_enrollment_Enrollment': '-1000,1000)',
        'MySU_models_enrollment_option_EnrollmentOption': '-1000,700)',
        'MySU_models_event_Event': '-1300,700)',
        'MySU_models_eventtype_EventType': '-1200,1000)',
        'MySU_models_group_Group': '-1600,500)',
        'MySU_models_group_membership_GroupMembership': '-1600,1000)',
        'MySU_models_institution_Institution': '300,2000)',
        'MySU_models_matching_Matching': '100,600)',
        'MySU_models_membership_fee_MembershipFee': '400,1000)',
        'MySU_models_membertype_MemberType': '400,600)',
        'MySU_models_news_News': '-300,0)',
        'MySU_models_notification_Notification': '-200,600)',
        'MySU_models_profile_Profile': '0,1700)',
        'MySU_models_study_Study': '300,1700)',
        'MySU_models_user_User': '-450,1700)',
        # 'MySU_models_study_record_StudyRecord': '300,2250)',
        # 'MySU_models_contact_info_ContactInfo': '0,0)',
        # 'MySU_models_permission_SUNPermission': '0,0)',
        # 'MySU_models_association_Society': '100,-250)',
        # 'computedfields_models_ComputedFieldsAdminModel': '1999,9999)',
        # 'computedfields_models_ComputedFieldsModel': '1999,9999)',
        # 'computedfields_models_ContributingModelsModel': '1999,9999)',
        # 'django_contrib_admin_models_LogEntry': '9999,9999)',
        # 'django_contrib_auth_base_user_AbstractBaseUser': '9999,9999)',
        'django_contrib_auth_models_Group': '-1600,750)',
        'django_contrib_auth_models_Permission': '-1800,750)',
        # 'django_contrib_auth_models_PermissionsMixin': '4999,9999)',
        # 'django_contrib_contenttypes_models_ContentType': '4999,9999)',
        # 'django_contrib_sessions_base_session_AbstractBaseSession': '3999,9999)',
        # 'django_contrib_sessions_models_Session': '3999,9999)',
        # 'django_cron_models_CronJobLog': '3999,9999)',
        # 'guardian_models_BaseGenericObjectPermission': '2999,9999)',
        # 'guardian_models_BaseObjectPermission': '2999,9999)',
        # 'guardian_models_GroupObjectPermission': '-1400,1300)',
        # 'guardian_models_GroupObjectPermissionBase': '2999,9999)',
        # 'guardian_models_UserObjectPermission': '2999,9999)',
        # 'guardian_models_UserObjectPermissionBase': '2999,9999)'
    }

    def add_arguments(self, parser):
        pass

    def handle(self, *args, **options):
        # Open a custom input/output stream to be able to read command line output.
        with io.StringIO() as out:
            # run the `graph_models` command from `django_extensions`
            management.call_command("graph_models",
                                    include_models=[
                                        "Association", "Notification", "AssociationMembership",
                                        "AssociationSpecificData", "AssociationSpecificDataFields",
                                        "DebtCollection", "DebtEntry", "Enrollment", "EnrollmentOption", "Enrollment",
                                        "Profile", "Study", "MembershipFee", "MemberType", "Event", "Group",
                                        "GroupMembership", "Institution", "Matching", "News",
                                        "Permission",
                                        # "User"
                                    ],
                                    hide_edge_labels=True,
                                    stdout=out,
                                    )
            # The `graph_models` command prints a .dot file to the input/output stream. Capture it in a variable.
            graph_dot_file = out.getvalue()
        # Create the graph using the .dot file.
        graph = pgv.AGraph(graph_dot_file)
        # Add location data to the nodes based on the node_position_dict
        for node in graph.nodes():
            node.attr["pos"] = self.node_position_dict[node.name]
        # Create and safe a .png with the graph, with some lines changed to curves to avoid drawing line under or over
        # models.
        graph.draw("UML.png", prog="neato", args="-n2")
