from rest_framework import serializers


class FilterRelatedMixin:
    """
    Mixin that enables filtering of related objects.

    The Browsable API usually shows all elements in the dropdowns for related objects even when those are not allowed to
    be shown. In the block below an example is shown in which the user is allowed to select any profile as
    preferred_profile even if it is not related.

    ```python
    class ProfileSerializer(serializers.HyperlinkedRelatedSerializer):
        class Meta:
            model = Profile
            fields = "__all__"

    class UserSerializer(serializers.HyperlinkedRelatedSerializer):
        class Meta:
            model = User
            fields = "__all__"
    ```
    When we add this Mixin and the associated filter function then this is no longer possible
    ```python
    class ProfileSerializer(serializers.HyperlinkedRelatedSerializer):
        class Meta:
            model = Profile
            fields = "__all__"

    class UserSerializer(FilterRelatedMixin, serializers.HyperlinkedRelatedSerializer):
        class Meta:
            model = User
            fields = "__all__"

        filter_preferred_profile(self, queryset):
            user = self.context['request'].user
            return user.profiles.all()
    ```

    Taken from https://github.com/encode/django-rest-framework/issues/1985#issuecomment-61871134
    """

    def __init__(self, *args, **kwargs):
        super(FilterRelatedMixin, self).__init__(*args, **kwargs)
        for name, field in iter(self.fields.items()):
            if isinstance(field, serializers.RelatedField):
                method_name = 'filter_%s' % name
                try:
                    func = getattr(self, method_name)
                except AttributeError:
                    pass
                else:
                    field.queryset = func(field.queryset)
            if isinstance(field, serializers.ManyRelatedField):
                method_name = 'filter_%s' % name
                try:
                    func = getattr(self, method_name)
                except AttributeError:
                    pass
                else:
                    field.child_relation.queryset = func(field.child_relation.queryset)
