import calendar
import datetime

from django.utils.deconstruct import deconstructible


def date_to_yearlessdate(date):
    return YearlessDate(date.month, date.day)


@deconstructible
class YearlessDate:
    """An object representing a date in a year (but not a year itself).  Suitable especially for birthdays, anniversaries etc."""

    def __init__(self, month, day):
        self.month = int(month)
        self.day = int(day)
        self._validate()

    def _validate(self):
        """Checks that the values of day and month are valid"""
        # Get valid max month days for the month (we use 2008 since it's a leap year)
        try:
            week, month_days = calendar.monthrange(2008, self.month)
        except calendar.IllegalMonthError:
            raise Exception('Cannot create DateInYear object with a month value of %d.' % self.month)

        if self.day < 1 or self.day > month_days:
            raise Exception(
                'Cannot create DateInYear object - invalid day value %d for month %s.' % (self.day, self.month_name))

    def with_year(self, year):
        return datetime.date(year, self.month, self.day)

    def with_year_in_future(self):
        today = datetime.date.today()
        date = datetime.date(today.year, self.month, self.day)
        if date < today:
            return datetime.date(today.year+1, self.month, self.day)
        else:
            return date

    @staticmethod
    def from_date(date):
        return YearlessDate(date.month, date.day)

    @staticmethod
    def following_each_other(followed, follows):
        return True

    @staticmethod
    def today():
        today = datetime.date.today()
        return YearlessDate(today.month, today.day)

    @property
    def month_name(self):
        """Returns the full name of the month"""
        return calendar.month_name[self.month]

    def __str__(self):
        return "{:02d}-{:02d}".format(self.month, self.day)

    def __repr__(self):
        return "yearlessdate({:02d},{:02d})".format(self.month, self.day)

    def __eq__(self, other):
        if isinstance(other, YearlessDate) or isinstance(other, datetime.date):
            return (self.day == other.day) and (self.month == other.month)
        return False

    def __gt__(self, other):
        if self.month != other.month:
            return self.month > other.month
        return self.day > other.day

    def __ne__(self, other):
        return not self == other

    def __le__(self, other):
        return not self > other

    def __ge__(self, other):
        return (self > other) or self == other

    def __lt__(self, other):
        return not self >= other

    def __sub__(self, other):
        # Difference depends on year
        year = datetime.date.today().year
        return self.with_year(year) - other.with_year(year)
