import logging

from django.conf import settings

from .google_api import GoogleApi


class GoogleAdmin:
    """
    This class is a wrapper around the GoogleApi class for the Admin SDK, i.e. the component of the Google API's that
    deals with creating user, groups and memberships. It provides the functions that we use in the models and the
    serializers relating to emails.

    To see the methods available to the underlying api, see:
       https://developers.google.com/admin-sdk/directory/reference/rest/v1/
       https://developers.google.com/resources/api-libraries/documentation/admin/directory_v1/python/latest/index.html
   """

    def __init__(self):
        self._api = None
        self.log = logging.getLogger("Gmail")

    @property
    def api(self):
        if self._api is None:
            self._api = GoogleApi.admin_sdk(
                scopes=[".user", ".group", ".group.member"]
            )
            self._api.with_service_account_file(
                settings.GOOGLE_API_ADMIN_SERVICE_ACCOUNT_FILE,
                settings.GOOGLE_API_ADMIN_SERVICE_ACCOUNT_EMAIL
            )
        return self._api

    # =========================== USERS ===========================

    def list_users(self, domain):
        """
        Lists all users from a given domain.

        Args:
            domain: The email domain (stuff behind the @ in the email).
        Returns:
            The list of users for the domain.
        """
        return self.api.users().list_all(domain=domain, key="users")

    def get_user(self, user_key):
        request = self.api.users().get(userKey=user_key)
        return request.execute()

    def add_user(self, username, password):
        """
        Adds a user.

        You can create a user passing an e-mail and a password. Example:
            {
                'kind': "admin#directory#user",
                'primaryEmail': "foo@bar.com",
                'password': "********"
            }
        Args:
            username: email and username for the user to create.
            password: password for the new user
        Returns:
            The created user
        """
        self.log.info('Adding user {}'.format(username))
        body = {
            "kind": "admin#directory#user",
            'primaryEmail': username,
            'password': password
        }
        request = self.api.users().add(body=body)
        return request.execute()

    def patch_user(self, user_key, user_data):
        """
        Patches a user

        Args:
            user_key: Immutable ID of the user that is to be patched
            user_data: dict with fields "username" and/or "password" that holds tells Google what values to update to what
        Returns:
            The patched user
        """
        self.log.info('Patching user {} with ID {}'.format(user_data["username"], user_key))
        # drops all keys but the primaryEmail and the password from the user_data
        body = {key: user_data[key] for key in {"username", "password"} & set(user_data.keys())}
        request = self.api.users().patch(userKey=user_key, body=body)
        return request.execute()

    def remove_user(self, user_key):
        self.log.info("Removing user {}".format(user_key))
        self.api.users().delete(user_key).execute()

    # =========================== GROUPS ===========================

    def list_groups(self, domain, *args, **kwargs):
        """
        Lists all groups from a given domain.

        Args:
            domain: The GSuite domain.
        Returns:
            The list of groups in the domain.
        """
        return self.api.groups().list_all(key="groups", domain=domain, *args, **kwargs)

    def add_group(self, group_data):
        """
        Add a group.

        You can create a group just passing the e-mail as string or an object with all the properties. Example:
            {
                'kind': "admin#directory#member",
                'description': "Foo",
                'email': "Bar",
                'name': "foo@bar.com"
            }
        Args:
            group_data: Group to create.
        Returns:
            The created group
        """
        self.log.info('Adding group {}'.format(group_data))
        if isinstance(group_data, dict):
            body = {
                "kind": "admin#directory#group",
                'description': group_data['description'],
                'email': group_data['email'],
                'name': group_data['name']
            }
        else:
            body = {
                "kind": "admin#directory#group",
                'description': '',
                'email': group_data,
                'name': group_data
            }
        req = self.api.groups().insert(body=body)
        return req.execute()

    def patch_group(self, group_key, email):
        """
        Patches a given group

        Args:
            group_key: group to be patched
            email: new email for the group
        Returns:
            The patches group
        """
        self.log.info("Patching group {}".format(group_key))
        body = {
            "email": email
        }
        request = self.api.groups().patch(groupKey=group_key, body=body)
        return request.execute()

    def remove_group(self, group_key):
        """
        Removes the specified group

        Args:
            group_key: Group to be removed
        """
        self.log.info("Removing group {}".format(group_key))
        self.api.groups().delete(groupKey=group_key).execute()

    # =========================== MEMBERS ===========================

    def list_members(self, group_key, *args, **kwargs):
        """
        Lists all members of a given group.

        Args:
            group_key: The group key to list all members. The value can be the group's email address, group alias, or the unique group ID.
        Returns:
            The list of membes in the group key. Members fields: id, email, type and role.
        """
        return self.api.members().list_all(key="members", groupKey=group_key, *args, **kwargs)

    def add_member_to_group(self, member, group_key):
        """
        Add a user to a group.

        You can inform a member just passing the e-mail as string ("role = MEMBER" by default) or
        an object with all the properties. Example:
            {
                'kind': "admin#directory#member",
                'role': "MANAGER / MEMBER / OWNER",
                'type': "CUSTOMER / EXTERNAL / GROUP / USER",
                'email': "foo@bar.com"
            }
        Args:
            member: Member to add in the group.
            group_key: The group key to add the member. The value can be the group's email address, group alias, or the unique group ID.
        Returns:
            The created membership
        """
        self.log.info('Adding member {} to group {}'.format(member, group_key))
        if isinstance(member, dict):
            body = {
                'kind': "admin#directory#member",
                'role': member['role'],
                'type': member['type'],
                'email': member['email']
            }
        else:
            body = {
                'kind': "admin#directory#member",
                'role': 'MEMBER',
                'email': member
            }
        request = self.api.members().insert(body=body, groupKey=group_key)
        return request.execute()

    def remove_member_from_group(self, member_key, group_key):
        """
        Removes a member from a group.

        Args:
            member_key: Member to be removed. A group member can be a user or another group. The value can be the member's (group or user) primary email address, alias, or unique ID.
            group_key: The group key. The value can be the group's email address, group alias, or the unique group ID.
        """
        self.log.info('Removing member {} from group {}'.format(member_key, group_key))
        self.api.members().delete(memberKey=member_key, groupKey=group_key).execute()
