import json
import logging
import time

from google.oauth2 import service_account
from googleapiclient.discovery import build
from googleapiclient.http import HttpError


class GoogleApi:
    """
    Google API helper object

    Heavily inspired on https://github.com/r00tat/google-api-helper
    """

    def __init__(self, api="oauth2", api_version="v2", scopes=['email'], *args, **kwargs):
        """constructor"""
        self.api = api
        self.api_version = api_version
        self.scopes = scopes
        self.credentials = kwargs.get('credentials')
        self.sub = kwargs.get('sub')
        self.retries = kwargs.get('retries', 3)
        self.log = logging.getLogger("GoogleApi")
        self._service = None

    @property
    def service(self):
        """
        Get or create the api service


        """
        if self._service is None:
            self._service = build(self.api, self.api_version, credentials=self.credentials)
        return self._service

    def with_service_account_file(self, service_account_file, sub=None):
        """
        Provides credentials for the service account to the API.

        The service account needs credentials. These can be constructed from data from a service account .json file.
        This function uses the tools provided by Google itself in order to derive the proper credentials from it, and
        stores them.
        """
        credentials = service_account.Credentials.from_service_account_file(service_account_file)
        if self.scopes:
            credentials = credentials.with_scopes(self.scopes)
        if sub:
            credentials = credentials.with_subject(sub)

        self.credentials = credentials
        self._service = None
        self.sub = sub
        return self

    def retry(self, service_method, retry_count=0):
        """
        Executes an api call and retries if necessary

        The google library behind the calls lets us execute a query by calling .execute() on the query. When an 403
        error occurs because we are doing to many requests a second, then the message contains "rate limit exceeded".
        When that happens we wait a few seconds and try again.
        """
        try:
            ret = service_method.execute(num_retries=retry_count)
        except HttpError as error:
            code = error.resp.get('code')

            reason = ''
            message = ''
            try:
                data = json.loads(error.content.decode('utf-8'))
                code = data['error']["code"]
                message = data['error']['message']
                reason = data['error']['errors'][0]['reason']
            except:  # noqa
                pass

            if code == 403 and "rate limit exceeded" in message.lower():
                self.log.info("rate limit reached, sleeping for %s seconds", 2**retry_count)
                time.sleep(2**retry_count)
                ret = self.retry(service_method, retry_count + 1)
            else:
                self.log.info("got http error {} ({}): {}".format(code, reason, message))
                raise
        except KeyboardInterrupt:
            raise
        except:  # noqa
            self.log.exception("Failed to execute api method")
            raise
        return ret

    def __getattr__(self, name):
        """
        Deligates getattr to an instance of MethodHelper.

        Args:
            name: attribute / service name
        Returns:
            Helper method or attribute
        """
        return getattr(MethodHelper(self, self.service), name)

    @classmethod
    def drive(cls, version="v3", scopes=[""]):
        """
        Drive v3 api
        """
        base_scope = "https://www.googleapis.com/auth/drive"
        scopes = [base_scope + scope for scope in scopes]
        return GoogleApi("drive", version, scopes)

    @classmethod
    def admin_sdk(cls, version="directory_v1", scopes=[".user"]):
        """
        Admin SDK v1
        """
        base_scope = "https://www.googleapis.com/auth/admin.directory"
        scopes = [base_scope + scope for scope in scopes]
        return GoogleApi("admin", version, scopes)

    @classmethod
    def gmail(cls, version="v1", scopes=[""]):
        """
        Gmail v1
        """
        base_scope = "https://mail.google.com/"
        scopes = [base_scope + scope for scope in scopes]
        return GoogleApi("gmail", version, scopes)

    @classmethod
    def calendar(cls, version="v3", scopes=[""]):
        """
        calendar v3
        """
        base_scope = "https://www.googleapis.com/auth/calendar"
        scopes = [base_scope + scope for scope in scopes]
        return GoogleApi("calendar", version, scopes)


class MethodHelper:
    """
    Helper to streamline api calls

    Without this helper all the calls of instances of the GoogleApi class would have to be called as
    self.api.service.service_method, e.g. self.api.service.users().list(). With the helper the service call will be done
    behind the scenes and hence we can simplify the call to the api to self.api.service_method, e.g.
    self.api.users().list().
    """

    def __init__(self, google_api, service, name=None, path=None):
        """
        Creates a method helper instance

        Args:
            google_api: GoogleApi instance of api
            service: Google API service (GoogleApi.service) or method of it
            name: method name
            path: API path i.e. for compute: instances.list
        """
        self.google_api = google_api
        self.service = service
        self.name = name
        self.path = path if path is not None else []
        if name is not None:
            self.path.append(name)

    def execute(self, *args, **kwargs):
        """execute service api"""
        return self.google_api.retry(self.service)

    def call(self, *args, **kwargs):
        """
        Wrapper for service methods

        This wraps an GoogleApi.service call so the next level can also use helpers
        i.e. for compute v1 api GoogleApi.service.instances() can be used as Google.instances()
        and will return a MethodHelper instance
        """
        return MethodHelper(self.google_api, getattr(self.service, self.name)(*args, **kwargs))

    def list_all(self, key="items", *args, **kwargs):
        """
        List all elements of a type without using pagination

        The list method normally returns just a set amount of results because of pagination. Sometimes this is
        inconvenient and you simply want all results. This method does this.

        Make sure you got enough memory to receive all elements. If pagination is an option, please use that over this
        helper function. (https://developers.google.com/api-client-library/python/guide/pagination)

        Args:
            key: key of the response dict containing the list of relevant items
        """
        elements = []
        request = self.service.list(**kwargs)
        while request is not None:
            response = self.google_api.retry(request)
            elements.extend(response.get(key, []))
            request = self.service.list_next(previous_request=request, previous_response=response)

        return elements

    def __getattr__(self, name):
        """ get service method """
        if not hasattr(self.service, name):
            err_msg = u"API method {} unknown on {} {}".format(u".".join(self.path + [name]),
                                                               self.google_api.api,
                                                               self.google_api.api_version)
            raise RuntimeError(err_msg)
        return MethodHelper(self.google_api, self.service, name, self.path).call
