"""
Django settings for MySU project.

For more information on this file, see
https://docs.djangoproject.com/en/dev/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/dev/ref/settings/
"""
import datetime
import os
import sys

from django.utils.translation import ugettext_lazy as _

"""
----------------------------------------------------------------------------------------------------------------------
IMPORT ENVIRONS
----------------------------------------------------------------------------------------------------------------------

Checks the system for a file containing the secret values used for production and develop
If the file with the environvalues does not exist, then settings defined in local.py or defaults will be used depending
on your hostname.
"""
environfile = '/var/www/sun/settings.py'

sys.path.append(os.path.dirname(os.path.expanduser(environfile)))

try:
    import settings
except ImportError:
    pass

"""
----------------------------------------------------------------------------------------------------------------------
SET ROOTS
----------------------------------------------------------------------------------------------------------------------

Django needs to know were to look for certain files. To build paths using BASE_DIR use 'os.path.join(BASE_DIR, ...)'
"""

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
PROJECT_ROOT = os.path.dirname(os.path.abspath(__name__))
sys.path.append(os.path.join(PROJECT_ROOT, "apps"))

ROOT_URLCONF = 'MySU.urls'

"""
----------------------------------------------------------------------------------------------------------------------
SET WSGI
----------------------------------------------------------------------------------------------------------------------

To let Django run a server it needs to know were the WSGI.py is located, it is can call it for running the server.
"""
WSGI_APPLICATION = 'MySU.wsgi.application'

"""
----------------------------------------------------------------------------------------------------------------------
SET APPS and MIDDLEWARE
----------------------------------------------------------------------------------------------------------------------

Every projects consists of several apps working together. To use an app in the project it needs to be listed here.
Middleware classes help the apps do their job. For the middleware to activated, it needs also to be included here.
"""

INSTALLED_APPS = [
    'dal',
    'dal_select2',
    'apps.buddy',
    'apps.calendar',
    'apps.MySU',
    'apps.financial',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.postgres',
    'django_filters',
    'rest_framework',
    'rest_framework_jwt',
    'drf_queryfields',
    'corsheaders',
    'guardian',
    'simple_history',
    'django_admin_listfilter_dropdown',
    'import_export',
    'drf_yasg',
    'rest_registration',
    'django_cron',
    'mozilla_django_oidc',
    "anymail",
    'multi_email_field',
    'computedfields',
    'colorfield',
    'ckeditor',
    'django_mail_template',
]

FIXTURE_DIRS = (
    'apps/MySU/fixtures',
    'apps/buddy/fixtures',
    'apps/calendar/fixtures',
    'apps/financial/fixtures',
)

MIDDLEWARE = [
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'simple_history.middleware.HistoryRequestMiddleware',
    'MySU.backends.jwt_authentication_middleware.JWTAuthenticationMiddleware'
]

CRON_CLASSES = [
    'MySU.cron.membership_period_end.MembershipPeriodEndCron',
]

"""
----------------------------------------------------------------------------------------------------------------------
CORS_HEADER CONFIG
----------------------------------------------------------------------------------------------------------------------

Settings for the app 'corsheader' is set here.
"""

CORS_ORIGIN_ALLOW_ALL = False
CORS_ORIGIN_WHITELIST = []
CORS_ALLOW_METHODS = (
    'DELETE',
    'GET',
    'OPTIONS',
    'PATCH',
    'POST',
    'PUT',
)

"""
----------------------------------------------------------------------------------------------------------------------
SET CONF FOR STATIC CONTENT
----------------------------------------------------------------------------------------------------------------------

The CSS, JavaScipt, Images and plain html need to have a dir and some URL to link to it. This is set here.
"""
MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
MEDIA_URL = 'media/'
STATIC_ROOT = os.path.join(BASE_DIR, 'static')
STATIC_URL = 'static/'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [BASE_DIR, os.path.join(BASE_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'MySU.context_processors.from_settings',
            ],
            'libraries': {
                'rest_framework_overwrites': 'MySU.templatetags.rest_framework_overwrites',
            }
        },
    },
]

"""
----------------------------------------------------------------------------------------------------------------------
SET CONF FOR AUTH
----------------------------------------------------------------------------------------------------------------------

The auth app is one of the most important apps in use in Django. It serves as portal for authentication and
authorization. To make sure the auth app is working settings are included here. Note that we do not use the
native user, but have defined a custom user.
"""

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
        'OPTIONS': {
            'min_length': 10,
        }
    },

    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },

]

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',  # this is default
    'guardian.backends.ObjectPermissionBackend',
    'MySU.backends.sun_oidc_backend.MySUOidcBackend',
)

AUTH_USER_MODEL = 'MySU.User'
DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'
"""
----------------------------------------------------------------------------------------------------------------------
SET CONF FOR REST FRAMEWORK
----------------------------------------------------------------------------------------------------------------------

The Rest Framework has many extensions. The ones configured here are the ones the Django will use.
"""

REST_FRAMEWORK = {
    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.DjangoModelPermissionsOrAnonReadOnly',
        'rest_framework.permissions.IsAuthenticated',
    ],
    'PAGE_SIZE': 10,
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework_jwt.authentication.JSONWebTokenAuthentication',
        'rest_framework.authentication.SessionAuthentication',
        'mozilla_django_oidc.contrib.drf.OIDCAuthentication',
    ),
    'DEFAULT_PAGINATION_CLASS': (
        'rest_framework.pagination.LimitOffsetPagination'
    ),
    'DEFAULT_FILTER_BACKENDS': (
        # 'django_filters.rest_framework.DjangoFilterBackend',
        'url_filter.integrations.drf.DjangoFilterBackend',
        'rest_framework.filters.OrderingFilter',
        'rest_framework.filters.SearchFilter',
    ),
    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework.renderers.JSONRenderer',
        'rest_framework.renderers.BrowsableAPIRenderer',
        'rest_framework_csv.renderers.CSVRenderer',
    ),
    'DEFAULT_SCHEMA_CLASS': 'rest_framework.schemas.coreapi.AutoSchema',
    'TEST_REQUEST_DEFAULT_FORMAT': 'json'
}
"""
----------------------------------------------------------------------------------------------------------------------
SET CONF FOR EMAIL
----------------------------------------------------------------------------------------------------------------------

"""
DEFAULT_FROM_EMAIL = 'MySU <no-reply@sun.utwente.nl>'
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

"""
----------------------------------------------------------------------------------------------------------------------
SET CONF FOR VERIFICATION EMAIL
----------------------------------------------------------------------------------------------------------------------

"""
ACCOUNT_ACTIVATION_DAYS = 7

"""
----------------------------------------------------------------------------------------------------------------------
SET CONF FOR LANGUAGES
----------------------------------------------------------------------------------------------------------------------

"""
LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

LOCALE_PATHS = [
    os.path.join(BASE_DIR, 'locale'),
]

LANGUAGES = (
    ('en', _('English')),
    ('nl', _('Dutch')),
)

"""
----------------------------------------------------------------------------------------------------------------------
SET OTHER CONF
----------------------------------------------------------------------------------------------------------------------

Here is config included that is here for good housekeeping and setting defaults for the settings files that include
this one.
"""
REST_REGISTRATION = {
    'REGISTER_VERIFICATION_ENABLED': False,
    'REGISTER_EMAIL_VERIFICATION_ENABLED': False,
    'RESET_PASSWORD_VERIFICATION_ENABLED': False,
    'VERIFICATION_FROM_EMAIL': 'no-reply@sun.utwente.nl',
    'USER_VERIFICATION_ID_FIELD': 'username',
}

DEBUG = False

INTERNAL_IPS = ['127.0.0.1', 'localhost']

JWT_AUTH = {
    'JWT_VERIFY_EXPIRATION': True,
    'JWT_ALLOW_REFRESH': True,
    'JWT_EXPIRATION_DELTA': datetime.timedelta(hours=4),
    'JWT_REFRESH_EXPIRATION_DELTA': datetime.timedelta(days=7),
}

IMPORT_EXPORT_USE_TRANSACTIONS = True

COMPUTEDFIELDS_ADMIN = True

"""
----------------------------------------------------------------------------------------------------------------------
OIDC (Surf LOGIN)
----------------------------------------------------------------------------------------------------------------------

"""

OIDC_RP_CLIENT_ID = os.environ.get('OIDC_RP_CLIENT_ID', None)
OIDC_RP_CLIENT_SECRET = os.environ.get('OIDC_RP_CLIENT_SECRET', None)

OIDC_RP_SIGN_ALGO = "RS256"
OIDC_ALLOW_UNSECURED_JWT = True
OIDC_DRF_AUTH_BACKEND = 'MySU.backends.sun_oidc_backend.MySUOidcBackend'
OIDC_AUTHENTICATE_CLASS = 'apps.MySU.views.MySUOidcAuthenticationRequestView'
OIDC_CREATE_USER = True
