import json

import tablib
from django.contrib import admin
from django.core.checks import messages
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.urls import path
from import_export.admin import ImportExportModelAdmin
from simple_history.admin import SimpleHistoryAdmin

from apps.MySU.models import Association
from apps.MySU.models import AssociationSpecificDataFields
from apps.MySU.models import MemberType
from apps.MySU.models import Profile
from apps.MySU.models import Study
from apps.MySU.models import User
from apps.MySU.serializers.mass_import import AssociationMemberImportSerializer
from MySU.forms.csv_import import CsvImportForm


@admin.register(Profile)
class ProfileAdmin(ImportExportModelAdmin, SimpleHistoryAdmin):
    # change_list_template = "django_admin/change_list.html"
    change_list_template = 'admin/import_export/change_list_import.html'
    # template for import view
    import_template_name = 'django_admin/import.html'
    csv_template_name = "django_admin/csv_form.html"

    # storage for data for confirm form
    data = None

    def get_urls(self):
        urls = super().get_urls()
        info = self.get_model_info()
        my_urls = [
            path('import/', self.admin_site.admin_view(self.import_csv), name='%s_%s_import' % info),
            path('import-process/', self.admin_site.admin_view(self.process_mass_import), name='%s_%s_mass_import_process' % info),
        ]
        return my_urls + urls

    def process_mass_import(self, request):
        if request.method == "POST" and self.data:
            # we know the data is correct, but many=True creation is not supported by the used package.
            # Hence we have to do this workaround of looping over the data.
            for (i, membership_data) in enumerate(self.data):
                request.data = membership_data
                serializer = AssociationMemberImportSerializer(data=membership_data, context={'request': request})
                if serializer.is_valid(raise_exception=False):
                    serializer.save()
                else:
                    self.message_user(request, "serializer on row {} gave errors: {}".format(i, json.dumps(serializer.errors, indent=2)), level="Error")
                    break
            self.message_user(request, "Import Succesful", level="Info")
            self.data = None
        else:
            self.message_user(request, "No import done", level="Info")
        return HttpResponseRedirect("..")

    def import_csv(self, request):
        self.data = None
        context = {}
        # if this is a POST request we need to process the form data
        if request.method == "POST":
            # create a form instance and populate it with data from the request:
            form = CsvImportForm(request.POST, request.FILES)
            if form.is_valid():
                association = form.cleaned_data['association']
                csv_file = form.cleaned_data['csv_file']
                delimiter = form.cleaned_data['delimiter']
                data = tablib.Dataset().load(csv_file.read().decode("utf-8-sig"), format='csv', delimiter=delimiter)
                json_data = self.make_serializable_json(request, data, association)
                if json_data:
                    serializer = AssociationMemberImportSerializer(data=json_data, many=True, context={'request': request})
                    valid = serializer.is_valid(raise_exception=False)
                    if valid:
                        table_data = self.make_table_data(serializer.validated_data)
                        self.data = json_data
                        # self.data = serializer.initial_data
                        context["table"] = table_data
                        context["confirm_form"] = form
                    else:
                        self.message_user(request, json.dumps(serializer.errors, indent=2), level="Error")
        # if a GET (or any other method) we'll create a blank form
        else:
            form = CsvImportForm()
        context["form"] = form
        context["title"] = "Import"
        context["fields"] = []
        context["opts"] = self.model._meta
        return render(request, self.import_template_name, context)

    def make_table_data(self, json_data):
        """
        In order to show a preview of the data it needs to be flattened first.

        This changes
        {
            "profile": {"profile_field1": "profile_value1",...,"profile_fieldN": "profile_valueN"},
            "date_joined": date_joined,
            "membertype": membertype,
            "specific_data": [
                {"field": field1, "value": value1, ...},
                ...,
                {"field": fieldN, "value": valueN, ...},
            ]
        }
        to
        {
            "profile_field1": "profile_value1",
            ...,
            "profile_fieldN": "profile_valueN",
            "date_joined": date_joined,
            "membertype": membertype,
            field1: value1,
            ...,
            fieldN: valueN,
        }
        """
        table_data = {
            "rows": [],
            "headers": []
        }
        for json_data_row in json_data:
            specific_data_dict = {data["data_field"].name: data["value"] for data in json_data_row["specific_data"]}
            # This adds all the keys with values of the nested profile and the specific data dict to json_data_row,
            # effectively flattening the data.
            intermediate_dict = {**json_data_row, **json_data_row["profile"], **specific_data_dict}
            # Ditch the nested profile and specific_data, the association
            intermediate_dict.pop("profile")
            intermediate_dict.pop("specific_data")
            intermediate_dict.pop("association")
            # update membertype to readable format
            intermediate_dict["type"] = intermediate_dict["type"].type
            # add only the values to the rows
            table_data["rows"].append(intermediate_dict.values())
            # set headers if not already set
            if not table_data["headers"]:
                table_data["headers"] = intermediate_dict.keys()
        return table_data

    def make_serializable_json(self, request, imported_data, association: Association):
        """
        This function takes the tablib csv file, does some validation on it and spits out the json version of it that
        can be passed to the serializers.

        :param request:
        :param imported_data:
        :param association:
        :return:
        """
        # WARNING: changing names below may break things.
        profile_only_fields = [
            'student_number',
            'given_name',
            'initials',
            'surname',
            'email',
            'phone_number',
            'address',
            'zip_code',
            'city',
            'country',
            'dutch_speaking',
            'gender',
            'date_of_birth',
            'bank_account_name',
            'iban',
            'bic',
            'phase',
            'study'
        ]
        membership_only_fields = [
            'membertype',
            'date_joined'
        ]

        #  ------------------------------------
        #  --- Validation based on headers  ---
        #  ------------------------------------

        # Check if the needed headers are present
        len_fields = len(profile_only_fields) + len(membership_only_fields)
        if imported_data.headers[:len_fields] != profile_only_fields + membership_only_fields:
            self.message_user(request, "Invalid HEADER format: {} \n should be : {}".format(
                imported_data.headers[:len_fields], profile_only_fields + membership_only_fields
            ), level=messages.ERROR)
            return

        #  -------------------------------------------------------------------------
        #  --- Separate data into data for profile, membership and specific data ---
        #  -------------------------------------------------------------------------

        # Separate the data for the profile in a new dataset
        dataset_profile = imported_data.subset(cols=profile_only_fields)
        # Add a column in back for the profile names. These are created to show the user which association imported the profile.
        dataset_profile.append_col(["Imported from {}".format(association.name)] * imported_data.height, header="profilename")
        # Add a column in back for the user foreignkey. These are used to match a user to a profile if that exists.
        dataset_profile.append_col(dataset_profile["student_number"], header="user")

        # Separate the data for the membership in a new dataset
        dataset_membership = imported_data.subset(cols=membership_only_fields)

        # For each membership the associated specific data instances needs to be created.
        if len(imported_data.headers[len_fields:]) != 0:
            # when there are specific data fields, separate the data for the membership in a new dataset
            dataset_specific_data = imported_data.subset(cols=imported_data.headers[len_fields:])
        else:
            # when there are no fields, create a dataset with the correct number of rows of empty lists.
            dataset_specific_data = [[] for _ in range(imported_data.height)]

        #  ------------------------------------------------------
        #  --- Retrieve relevant instances from the database  ---
        #  ------------------------------------------------------

        # Fetch the related specific data fields from the database
        association_specific_fields = list(AssociationSpecificDataFields.objects.filter(association=association).order_by("name"))
        # Fetch the related membertypes from the database
        membertypes = list(MemberType.objects.filter(association=association))
        # Fetch the matching users from the database
        users = list(User.objects.filter(student_number__in=dataset_profile['student_number']))
        # Map to dict for easy access
        user_student_number_to_slug = {user.student_number: str(user.slug) for user in users}
        # Fetch the studies
        studies = list(Study.objects.all())
        # Map to dict for easy access
        study_name_to_slug = {study.name: str(study.slug) for study in studies}

        #  -----------------------------------------------------------------------
        #  --- Validate values based on instances retrieved from the database  ---
        #  -----------------------------------------------------------------------

        # We are going to validate the headers of the fields first.
        # Gather the specific data headers
        association_specific_fields_headers = imported_data.headers[len_fields:]
        # It is easier if we focus just on the names, so we map the fields to a list with the names.
        existing_specific_field_names = [field.name for field in association_specific_fields]
        # Headers for association specific fields must be sorted alphabetically. Enforcing this here, makes life easier later
        if sorted(association_specific_fields_headers) != association_specific_fields_headers:
            self.message_user(request, "I0: Specific data field headers should be in alphabetical order.", level=messages.ERROR)
            return

        # get the list of the supplied headers that are in the fields
        headers_in_fields = [field for field in association_specific_fields_headers if field in existing_specific_field_names]
        # get the list of the supplied headers that are contained in the fields
        fields_in_headers = [field for field in existing_specific_field_names if field in association_specific_fields_headers]
        # validate the fields
        if headers_in_fields != association_specific_fields_headers:
            self.message_user(request, "I1a: A specific data field headers is not a valid field in the database.", level=messages.ERROR)
            return
        if fields_in_headers != existing_specific_field_names:
            self.message_user(request, "I1b: Not all specific data fields are supplied as header", level=messages.ERROR)
            return

        # We are going to validate the membertypes. This is easier if we focus just on the names, so we map the types to
        # a list with the names.
        existing_membertype_names = [membertype.type for membertype in membertypes]
        provided_membertype_names = dataset_membership['membertype']
        for (m, membertype) in enumerate(provided_membertype_names):
            if membertype not in existing_membertype_names:
                self.message_user(request, "I2: Membertype {} on row {} not a valid membertype".format(membertype, m), level=messages.ERROR)
                return

        #  ------------------------------------------
        #  --- Update values using the instances  ---
        #  ------------------------------------------

        # when creating we need instances not string, so we map these.
        # create dict to make update of membertypes easier
        membertype_dict = {membertype.type: membertype for membertype in membertypes}

        # Boolean values can be entered in many ways, so we need to update the dataset
        # The dataset is immutable so we create a fresh one
        updated_dataset_specific_data = tablib.Dataset()
        if association_specific_fields:
            for (f, field) in enumerate(association_specific_fields):
                data_row = dataset_specific_data[field.name]
                updated_row = []
                for (d, data) in enumerate(data_row):
                    if field.type[0] == "Boolean":
                        if data in ['Yes', 'yes', 'true', 'True', True, '1', 'Ja', 'ja', 'wel']:
                            updated_row.append("True")
                        elif data in ['No', 'no', 'false', 'False', False, '0', 'Nee', 'nee', 'niet']:
                            updated_row.append("False")
                        else:
                            self.message_user(request, "I4: Invalid Data {} found in column {} on row {}".format(data, field.name, d), level=messages.ERROR)
                            return
                    else:
                        updated_row.append(data)
                # add column back to dataset
                updated_dataset_specific_data.append_col(updated_row, header=field.name)
        else:
            # To make sure that the for loop at the end works, we need to ensure that at least one element on each row
            # exists. Thus we create an empty row for membership.
            updated_dataset_specific_data.append_col([[] * dataset_profile.height], header="dummy header name")

        # study will be entered by their unique english name and not
        subset_headers = list(dataset_profile.headers)
        subset_headers.remove("study")
        updated_dataset_profile = dataset_profile.subset(cols=subset_headers)
        # get the data from the study column
        study_column = dataset_profile["study"]
        new_study_column = []
        for (s, study_name) in enumerate(study_column):
            try:
                if study_name:
                    new_study_column.append("/studies/" + study_name_to_slug[study_name])
                else:
                    new_study_column.append(None)
            except KeyError:
                self.message_user(request, "I5: Invalid Study {} found on row {}".format(study_name, s), level=messages.ERROR)
                return
        # add column back to dataset
        updated_dataset_profile.append_col(new_study_column, header="study")

        nested_json_dataset = []
        for profile, membership, specific_data in zip(updated_dataset_profile, dataset_membership, updated_dataset_specific_data):
            try:
                user = "/users/" + user_student_number_to_slug[profile[0]]
            except KeyError:
                user = ""
            toBeCreatedProfile = {field: value for (field, value) in zip(updated_dataset_profile.headers, profile)}
            toBeCreatedProfile["bank_account"] = {
                "name": toBeCreatedProfile["bank_account_name"],
                "iban": toBeCreatedProfile["iban"],
                "bic": toBeCreatedProfile["bic"],
            }
            nested_json_dataset.append({
                "user": user,
                "profile": toBeCreatedProfile,
                "type": "/membertypes/" + str(membertype_dict[membership[0]].slug),
                "status": "Claimed",
                "date_joined": membership[1],
                "association": "/associations/" + association.slug,
                "specific_data": [
                    {"value": data,
                     "data_field": "/association_data_fields/" + str(field.slug),
                     "association": "/associations/" + association.slug,
                     # "name": field.name
                     } for (data, field) in zip(specific_data, association_specific_fields)
                ]
            })
        return nested_json_dataset

    list_display = (
        'slug',
        'user',
        'student_number',
        'given_name',
        'initials',
        'surname',
        'profilename'
    )

    search_fields = (
        'user__username',
        'given_name',
        'initials',
        'surname',
    )

    list_filter = (
        'profilename',
    )
