import datetime
import uuid

from autoslug import AutoSlugField
from colorfield.fields import ColorField
from django.core.exceptions import ValidationError
from django.core.validators import MinValueValidator
from django.db import models
from django.db.models import Q
from simple_history.models import HistoricalRecords

from apps.MySU.models.contact_info import ContactInfo
from MySU.fields.yearlessdate import YearlessDateField


class AssociationQuerySet(models.QuerySet):
    pass


class AssociationManager(models.Manager):
    def get_queryset(self):
        return AssociationQuerySet(self.model, using=self._db)

    def associations_where_user_is_member_on_date(self, user: 'MySU.User', date):
        return self \
            .filter(Q(users=user) & (Q(memberships__date_left__lt=date) | Q(memberships__date_left__isnull=True)))

    def associations_where_user_is_member(self, user: 'MySU.User'):
        return self \
            .filter(users=user, memberships__current=True)


class Association(ContactInfo):
    """
    Main organizational unit. Every association that has joined My SU will have one Association entity in the
    database. Almost all models will have a connection to this.
    """
    slug = AutoSlugField(default=uuid.uuid4, editable=False, max_length=200, populate_from='name')
    name = models.CharField(max_length=200)
    short_name = models.CharField(max_length=200)
    founding_date = models.DateField(default=datetime.date.today)
    description = models.TextField(blank=True)
    new_member_title = models.TextField(blank=True)
    new_member_text = models.TextField(blank=True)
    full_year_membership_start_date = YearlessDateField()
    photo = models.ImageField(upload_to='association/photo/', blank=True)
    logo = models.ImageField(upload_to='association/logo/', blank=True)
    bylaws_en = models.FileField(upload_to='association/bylaws/en/', blank=True)
    bylaws_nl = models.FileField(upload_to='association/bylaws/nl/', blank=True)
    articles_of_association_en = models.FileField(upload_to='association/articles_of_association/en/', blank=True)
    articles_of_association_nl = models.FileField(upload_to='association/articles_of_association/nl/', blank=True)
    privacy_statement = models.FileField(upload_to='association/privacy_statement/', blank=True)
    terms_of_service = models.FileField(upload_to='association/terms_of_service/', blank=True)
    dissolution_date = models.DateField(blank=True, null=True)
    room = models.CharField(max_length=100, blank=True)
    postal_address = models.CharField(max_length=200, blank=True)
    postal_zipcode = models.CharField(max_length=10, blank=True)
    postal_city = models.CharField(max_length=50, blank=True)
    whatsapp = models.CharField(max_length=20, blank=True)
    discord = models.CharField(max_length=100, blank=True)
    instagram = models.CharField(max_length=100, blank=True)
    facebook = models.CharField(max_length=100, blank=True)
    snapchat = models.CharField(max_length=100, blank=True)
    linkedin = models.CharField(max_length=100, blank=True)
    color = ColorField(default="#3f51b5")
    # Financial settings
    bank_account_name = models.CharField(max_length=100, blank=True, default="")
    iban = models.CharField(max_length=100, blank=True, default="")
    bic = models.CharField(max_length=100, blank=True, default="")
    incassant_id = models.CharField(max_length=200, blank=True, default="")
    debt_collection_threshold = models.DecimalField(validators=[MinValueValidator(0.0)], decimal_places=2, max_digits=5, default=0)
    member_needs_iban = models.BooleanField(default=True)
    # Email settings
    send_email_on_create_matching = models.BooleanField(default=True)
    email = models.EmailField()
    # DEFAULT_MEMBERSHIP_ACCEPTED_EMAIL_SUBJECT = ""
    # membership_accepted_email_subject = models.CharField(max_length=100, default=DEFAULT_MEMBERSHIP_ACCEPTED_EMAIL_SUBJECT)
    # DEFAULT_MEMBERSHIP_ACCEPTED_EMAIL_MESSAGE = ""
    # membership_accepted_email_message = models.TextField(default=DEFAULT_MEMBERSHIP_ACCEPTED_EMAIL_MESSAGE)
    # DEFAULT_MEMBERSHIP_REJECTED_EMAIL_SUBJECT = ""
    # membership_rejected_email_subject = models.CharField(max_length=100, default=DEFAULT_MEMBERSHIP_REJECTED_EMAIL_SUBJECT)
    # DEFAULT_MEMBERSHIP_REJECTED_EMAIL_MESSAGE = ""
    # membership_rejected_email_message = models.TextField(default=DEFAULT_MEMBERSHIP_REJECTED_EMAIL_MESSAGE)
    # DEFAULT_MEMBERSHIP_END_EMAIL_SUBJECT = ""
    # membership_end_email_subject = models.CharField(max_length=100, default=DEFAULT_MEMBERSHIP_END_EMAIL_SUBJECT)
    # DEFAULT_MEMBERSHIP_END_EMAIL_MESSAGE = ""
    # membership_end_email_message = models.TextField(default=DEFAULT_MEMBERSHIP_END_EMAIL_MESSAGE)
    # DEFAULT_GROUP_MEMBERSHIP_ADDED_EMAIL_SUBJECT = ""
    # group_membership_added_email_subject = models.CharField(max_length=100, default=DEFAULT_GROUP_MEMBERSHIP_ADDED_EMAIL_SUBJECT)
    # DEFAULT_GROUP_MEMBERSHIP_ADDED_EMAIL_MESSAGE = ""
    # group_membership_added_email_message = models.TextField(default=DEFAULT_GROUP_MEMBERSHIP_ADDED_EMAIL_MESSAGE)
    # DEFAULT_GROUP_MEMBERSHIP_REMOVED_EMAIL_SUBJECT = ""
    # group_membership_removed_email_subject = models.CharField(max_length=100, default=DEFAULT_GROUP_MEMBERSHIP_REMOVED_EMAIL_SUBJECT)
    # DEFAULT_GROUP_MEMBERSHIP_REMOVED_EMAIL_MESSAGE = ""
    # group_membership_removed_email_message = models.TextField(default=DEFAULT_GROUP_MEMBERSHIP_REMOVED_EMAIL_MESSAGE)
    # DEFAULT_GROUP_MEMBERSHIP_ENDED_EMAIL_SUBJECT = ""
    # group_membership_ended_email_subject = models.CharField(max_length=100, default=DEFAULT_GROUP_MEMBERSHIP_ENDED_EMAIL_SUBJECT)
    # DEFAULT_GROUP_MEMBERSHIP_ENDED_EMAIL_MESSAGE = ""
    # group_membership_ended_email_message = models.TextField(default=DEFAULT_GROUP_MEMBERSHIP_ENDED_EMAIL_MESSAGE)
    # DEFAULT_EVENT_ENROLLED_EMAIL_SUBJECT = ""
    # event_enrolled_email_subject = models.CharField(max_length=100, default=DEFAULT_EVENT_ENROLLED_EMAIL_SUBJECT)
    # DEFAULT_EVENT_ENROLLED_EMAIL_MESSAGE = ""
    # event_enrolled_email_message = models.TextField(default=DEFAULT_EVENT_ENROLLED_EMAIL_MESSAGE)
    # DEFAULT_EVENT_UNENROLLED_EMAIL_SUBJECT = ""
    # event_unenrolled_email_subject = models.CharField(max_length=100, default=DEFAULT_EVENT_UNENROLLED_EMAIL_SUBJECT)
    # DEFAULT_EVENT_UNENROLLED_EMAIL_MESSAGE = ""
    # event_unenrolled_email_message = models.TextField(default=DEFAULT_EVENT_UNENROLLED_EMAIL_MESSAGE)
    DEFAULT_MATCH_CREATED_EMAIL_SUBJECT = "You have been matched"
    match_created_email_subject = models.CharField(max_length=100, default=DEFAULT_MATCH_CREATED_EMAIL_SUBJECT)
    DEFAULT_MATCH_CREATED_EMAIL_MESSAGE = "Hi {matched}\n\nYou have been matched with {matched_with.name} by {association}. You can contact your match by email using {matched_with.email}.\n\nKind regards,\n\nThe MySU team"
    match_created_email_message = models.TextField(default=DEFAULT_MATCH_CREATED_EMAIL_MESSAGE)

    members = models.ManyToManyField('Profile', through='AssociationMembership', related_name='associations')
    users = models.ManyToManyField('User', through='AssociationMembership', related_name='associations')

    # @property
    # def default_membership_accepted_email_subject(self):
    #     return self.DEFAULT_MEMBERSHIP_ACCEPTED_EMAIL_SUBJECT
    #
    # @property
    # def default_membership_accepted_email_message(self):
    #     return self.DEFAULT_MEMBERSHIP_ACCEPTED_EMAIL_MESSAGE
    #
    # @property
    # def default_membership_rejected_email_subject(self):
    #     return self.DEFAULT_MEMBERSHIP_REJECTED_EMAIL_SUBJECT
    #
    # @property
    # def default_membership_rejected_email_message(self):
    #     return self.DEFAULT_MEMBERSHIP_REJECTED_EMAIL_MESSAGE
    #
    # @property
    # def default_membership_end_email_subject(self):
    #     return self.DEFAULT_MEMBERSHIP_END_EMAIL_SUBJECT
    #
    # @property
    # def default_membership_end_email_message(self):
    #     return self.DEFAULT_MEMBERSHIP_END_EMAIL_MESSAGE
    #
    # @property
    # def default_group_membership_added_email_subject(self):
    #     return self.DEFAULT_GROUP_MEMBERSHIP_ADDED_EMAIL_SUBJECT
    #
    # @property
    # def default_group_membership_added_email_message(self):
    #     return self.DEFAULT_GROUP_MEMBERSHIP_ADDED_EMAIL_MESSAGE
    #
    # @property
    # def default_group_membership_removed_email_subject(self):
    #     return self.DEFAULT_GROUP_MEMBERSHIP_REMOVED_EMAIL_SUBJECT
    #
    # @property
    # def default_group_membership_removed_email_message(self):
    #     return self.DEFAULT_GROUP_MEMBERSHIP_REMOVED_EMAIL_MESSAGE
    #
    # @property
    # def default_event_enrolled_email_subject(self):
    #     return self.DEFAULT_EVENT_ENROLLED_EMAIL_SUBJECT
    #
    # @property
    # def default_event_enrolled_email_message(self):
    #     return self.DEFAULT_EVENT_ENROLLED_EMAIL_MESSAGE
    #
    # @property
    # def default_event_unenrolled_email_subject(self):
    #     return self.DEFAULT_EVENT_UNENROLLED_EMAIL_SUBJECT
    #
    # @property
    # def default_event_unenrolled_email_message(self):
    #     return self.DEFAULT_EVENT_UNENROLLED_EMAIL_MESSAGE
    #
    @property
    def default_match_created_email_subject(self):
        return self.DEFAULT_MATCH_CREATED_EMAIL_SUBJECT

    @property
    def default_match_created_email_message(self):
        return self.DEFAULT_MATCH_CREATED_EMAIL_MESSAGE

    objects = AssociationManager()

    class Meta:
        ordering = ["name"]
        verbose_name_plural = "Associations"
        permissions = (
            ("board", "Board"),
        )

    def __str__(self):
        return self.name

    def clean(self):
        if self.dissolution_date:
            if self.dissolution_date < self.founding_date:
                raise ValidationError("Dissolution must occur after founding")


class Society(Association):
    # ... Add extra fields depending on if necessary

    class Meta:
        ordering = ["name"]
        verbose_name = "Society"
        verbose_name_plural = "Societies"
        permissions = (
            ("society", "Society"),
        )
