import datetime
import uuid

from computedfields.models import computed
from computedfields.models import ComputedFieldsModel
from django.conf import settings
from django.core.exceptions import ValidationError
from django.core.mail import send_mail
from django.db import models
from django.db.models import F
from django.db.models import Q

from apps.MySU.models import Association
from apps.MySU.models import MemberType
from apps.MySU.models import Profile


class AssociationMembershipQuerySet(models.QuerySet):
    def current(self):
        return self.filter(current=True)

    def filter_on_period_ended(self):
        today = datetime.date.today()
        yesterday = today + datetime.timedelta(-1)
        return self.filter(
            Q(type__full_year_membership=True, association__full_year_membership_start_date=today) |
            Q(type__full_year_membership=False, type__end_date=yesterday)
        )

    def active(self):
        return self.filter(profile__group_member__association=F('association'),
                           profile__group_member__date_left__isnull=True)


class AssociationMembershipManager(models.Manager):
    def get_queryset(self):
        return AssociationMembershipQuerySet(self.model, using=self._db)

    def current(self):
        return self.get_queryset().current()

    def filter_on_period_ended(self):
        return self.get_queryset().filter_on_period_ended()

    def active(self):
        return self.get_queryset().active()

    def is_current_member(self, profile, association):
        return self.get_queryset().current().filter(association=association, profile=profile).exists()


class AssociationMembership(ComputedFieldsModel):
    """
    Link between an association and a profile. This model encodes the relevant information regarding this link.

    # status
    The meaning of each of the status options is listed below. This should be read as:
    {status}    : {explanation}

    Accepted    : The user of the profile has logged in and confirmed to be a member of the association
    Claimed     : The association imported the data from the its own local database into My SU, and the imported user
                  has not logged in to confirm its membership.
    Archived    : This person has left the association, but wishes that contact information remained with them after
                  ending the membership at the association.
    Disputed    : The user of the profile has logged in, but did not confirm to be a member of the association.
                  This means there is a dispute about whether the user is actually a member of the association and
                  whether this data should be visible to the association
    Ended       : This is a membership that has ended. It will no longer be visible to the association, but will still
                  be visible to the user.
    Pending     : A user has requested to be a member, but the association has not Accepted nor Rejected the
                  request yet.
    Rejected    : A user has requested to be a member, but the association for some reason or another has Rejected the
                  request.

    The allowed status changes are listed below. This should be read as:
    {from status} -> {to status}
    An empty {to status} means that it can be removed, when it is in that state.
    A {to status} equal to ~ means that it can be removed but removing means autonomizing it, when it is in that state.

    Accepted    -> Ended
                -> Archived
    Claimed     -> Accepted
                -> Disputed
                -> Ended
                -> Archived
    Archived    -> ~
    Disputed    -> Accepted
                -> Claimed*
    Ended       -> ~
    Pending     -> Accepted
                -> Rejected
                ->
    Rejected    ->

    * When there is a dispute and the association cannot figure it out with the person, then an My SU admin should
    intervene and resolve matters manually.

    The terminal state of membership is an autonomized version of the Archived or Ended state, or deletion. Deletion can
    only occur for memberships that have never been, i.e. a rejection or cancelled membership.

    These allowed status changes imply that the status flows listed below are possible:
     Pending ->
     Pending -> Rejected ->
     Pending -> Accepted -> Ended -> ~
     Pending -> Accepted -> Archived -> ~
     Pending -> Accepted -> Archived -> Ended -> ~
     Claimed -> Accepted -> Ended -> ~
     Claimed -> Accepted -> Archived -> ~
     Claimed -> Accepted -> Archived -> Ended -> ~
     Claimed -> Disputed -> Accepted -> Ended -> ~
     Claimed -> Disputed -> Accepted -> Archived -> ~
     Claimed -> Disputed -> Accepted -> Archived -> Ended -> ~
     Claimed -> Disputed -> Claimed -> ....

    # financial_obligations_satisfied
    When a membership has ended, it can happen that the member has not fully paid the association yet. This field
    indicates that. After a membership has ended and the association has indicated that the members financial obligations
    have been satisfied, then the association will no longer be able to see the data of the member.

    # visible_after_date_left
    This field determines whether once the membership ends it will become Archived or Ended. When this field is changed
    after it has become Archived or Ended, then the status should be changed simultaneously.

    # new_type & change_type_on_period_end
    When a member period has ended, change_type_on_period_end determines whether for the new period the user should have
    a new member type or keep the same. If change_type_on_period_end is true, the new_type will be used as the member
    type for the new period.
    """

    class Status(models.TextChoices):
        ACCEPTED = "Accepted", "Accepted"
        CLAIMED = "Claimed", "Claimed"
        ARCHIVED = "Archived", "Archived"
        DISPUTED = "Disputed", "Disputed"
        ENDED = "Ended", "Ended"
        PENDING = "Pending", "Pending"
        REJECTED = "Rejected", "Rejected"

    class PayBy(models.TextChoices):
        INVOICE = "Invoice", "Invoice"
        DEBT_COLLECTION = "Debt collection", "Debt collection"

    slug = models.UUIDField(default=uuid.uuid4, editable=False)
    last_modified = models.DateTimeField(auto_now=True)
    status = models.CharField(max_length=20, choices=Status.choices, default=Status.PENDING)
    pay_by = models.CharField(max_length=20, choices=PayBy.choices, default=PayBy.DEBT_COLLECTION)
    date_joined = models.DateField(default=datetime.date.today)
    date_left = models.DateField(null=True, blank=True)
    financial_obligations_satisfied = models.BooleanField(default=False, blank=True)
    visible_after_date_left = models.BooleanField(default=False, blank=True)
    change_type_on_period_end = models.BooleanField(default=False, blank=True)
    accepted_email_send = models.BooleanField(default=False, blank=True)
    ended_email_send = models.BooleanField(default=False, blank=True)
    rejected_email_send = models.BooleanField(default=False, blank=True)
    confirmation_of_request_to_end_email_send = models.BooleanField(default=False, blank=True)
    financial_obligations_satisfied_email_send = models.BooleanField(default=False, blank=True)

    profile = models.ForeignKey(Profile, related_name='association_memberships', on_delete=models.PROTECT)
    sepa_mandate = models.ForeignKey('financial.SepaMandate', on_delete=models.PROTECT, null=True, blank=True)
    user = models.ForeignKey('MySU.User', on_delete=models.PROTECT, null=True, blank=True)
    type = models.ForeignKey(MemberType, on_delete=models.PROTECT, related_name='memberships')
    new_type = models.ForeignKey(MemberType, on_delete=models.PROTECT, related_name='new_memberships', null=True,
                                 blank=True)
    association = models.ForeignKey(Association, on_delete=models.PROTECT, related_name='memberships', null=False,
                                    blank=False)
    data_fields = models.ManyToManyField('AssociationSpecificDataFields', through='AssociationSpecificData',
                                         related_name='association_memberships')

    @computed(models.BooleanField(default=False), depends=[['self', ['status']]])
    def current(self):
        return self.status in [self.Status.ACCEPTED, self.Status.DISPUTED, self.Status.CLAIMED]

    @computed(models.DateTimeField(default=datetime.date.today), depends=[['specific_data', ['last_modified']]])
    def data_fields_last_modified(self):
        return max([data_field.last_modified for data_field in self.specific_data.all()], default=datetime.datetime.min)

    objects = AssociationMembershipManager()

    class Meta:
        verbose_name = "AssociationMembership"
        verbose_name_plural = "AssociationMemberships"
        constraints = [
            models.UniqueConstraint(fields=['profile', 'association', 'date_left'],
                                    name="memberships_unique_with_date_left"),
            models.UniqueConstraint(fields=['profile', 'association'],
                                    condition=models.Q(date_left=None),
                                    name="memberships_unique_without_date_left")
        ]

    def __str__(self):
        return "{slug}".format(slug=self.slug)

    def save(self, *args, **kwargs):
        if self.date_left and not self.confirmation_of_request_to_end_email_send:
            self.send_confirmation_of_request_to_end_email(self.profile, self.association)
            self.confirmation_of_request_to_end_email_send = True
        if not self.date_left and self.confirmation_of_request_to_end_email_send:
            self.confirmation_of_request_to_end_email_send = False
        if self.status == self.Status.ACCEPTED and not self.accepted_email_send:
            self.send_accepted_email(self.profile, self.association)
            self.accepted_email_send = True
        if self.status == self.Status.ENDED and not self.ended_email_send:
            self.send_ended_email(self.profile, self.association)
            self.ended_email_send = True
        if self.financial_obligations_satisfied and not self.financial_obligations_satisfied_email_send:
            self.send_financial_obligations_satisfied_email(self.profile, self.association)
            self.financial_obligations_satisfied_email_send = True
        if self.status == self.Status.REJECTED and not self.rejected_email_send:
            self.send_rejected_email(self.profile, self.association)
            self.rejected_email_send = True
        super(AssociationMembership, self).save(*args, **kwargs)

    @staticmethod
    def send_confirmation_of_request_to_end_email(profile, association):
        from_email = settings.DEFAULT_FROM_EMAIL
        to_email = [profile.email]
        subject = f"Confirmation of request to end membership of {association.short_name}"
        message = f"Hi {profile.given_name}\n\n" \
                  f"You have indicated that you no longer want to be a member of {association.name}. Your membership will end by " \
                  f"{association.full_year_membership_start_date.with_year_in_future()}. Afterwards your data will still be visible to the board until " \
                  f"they have indicated that you have fulfilled your financial obligations. You will be informed when they do." \
                  f"Kind regards\n\n" \
                  f"The MySU team\n\n" \
                  f"This is an automatically generated email. Please contact the board of {association.name} if you have any questions."
        send_mail(subject=subject, message=message, from_email=from_email, recipient_list=to_email)

    @staticmethod
    def send_financial_obligations_satisfied_email(profile, association):
        from_email = settings.DEFAULT_FROM_EMAIL
        to_email = [profile.email]
        subject = f"Membership of {association.short_name} accepted"
        message = f"Hi {profile.given_name}\n\n" \
                  f"When you ended your membership you received an email that informed you the board will still be able to see " \
                  f"your data until you have fulfilled your financial obligations. The {association.name} has now indicated you have " \
                  f"fulfilled all your financial obligations.\n\n" \
                  f"Your data will no longer be visible to {association.name}.\n\n" \
                  f"Kind regards\n\n" \
                  f"The MySU team\n\n" \
                  f"This is an automatically generated email. Please contact the board of {association.name} if you have any questions."
        send_mail(subject=subject, message=message, from_email=from_email, recipient_list=to_email)

    # pylint: disable=arguments-differ
    @staticmethod
    def send_ended_email(profile, association):
        from_email = settings.DEFAULT_FROM_EMAIL
        to_email = [profile.email]
        subject = f"Membership of {association.short_name} ended"
        message = f"Hi {profile.given_name}\n\n" \
                  f"Your membership with {association.name} has ended. \n\n" \
                  f"Once the board of {association.name} has indicated that you have fulfilled all your financial obligations, " \
                  f"they will no longer be able to see your data. You will be informed when they do.\n\n" \
                  f"Kind regards\n\n" \
                  f"The MySU team\n\n" \
                  f"This is an automatically generated email. Please contact the board of {association.name} if you have any questions."
        send_mail(subject=subject, message=message, from_email=from_email, recipient_list=to_email)

    @staticmethod
    def send_accepted_email(profile, association):
        from_email = settings.DEFAULT_FROM_EMAIL
        to_email = [profile.email]
        subject = f"Membership of {association.short_name} accepted"
        message = f"Hi {profile.given_name}\n\n" \
                  f"Your membership request for {association.name} has been accepted.\n\n" \
                  f"Kind regards\n\n" \
                  f"The MySU team\n\n" \
                  f"This is an automatically generated email. Please contact the board of {association.name} if you have any questions."
        send_mail(subject=subject, message=message, from_email=from_email, recipient_list=to_email)

    @staticmethod
    def send_rejected_email(profile, association):
        from_email = settings.DEFAULT_FROM_EMAIL
        to_email = [profile.email]
        subject = f"Membership of {association.short_name} rejected"
        message = f"Hi {profile.given_name}\n\n" \
                  f"Your membership request for {association.name} has been rejected.\n\n" \
                  f"Kind regards\n\n" \
                  f"The MySU team\n\n" \
                  f"This is an automatically generated email. Please contact the board of {association.name} if you have any questions."
        send_mail(subject=subject, message=message, from_email=from_email, recipient_list=to_email)

    # @staticmethod
    # def send_email(profile, association):
    #     subject = "Membership of {association} accepted".format(association=association.short_name)
    #     from_email = settings.DEFAULT_FROM_EMAIL
    #     to_email = [profile.email]
    #
    #     message = "Hi {}\n\nYour membership request for {} has been accepted.\n\nKind regards\n\nThe MySU team"\
    #         .format(profile.given_name, association.name)
    #     # template_html = 'association_membership_conformation_email.html'
    #     # template_text = 'association_membership_conformation_email.txt'
    #     # text_content = render_to_string(template_text,
    #     #                                 {"name": profile.given_name, "association": association})
    #     # html_content = render_to_string(template_html,
    #     #                                 {"name": profile.given_name, "association": association})
    #     #
    #     # send_mail(subject=subject, message=text_content, html_message=html_content, from_email=from_email,
    #     #           recipient_list=to_email, fail_silently=False)
    #     send_mail(subject=subject, message=message, from_email=from_email, recipient_list=to_email)
