import uuid

from django.core.exceptions import ValidationError
from django.db import models

from apps.MySU.models import Association
from apps.MySU.models import AssociationMembership
from apps.MySU.models import AssociationSpecificDataFields


class AssociationSpecificDataQuerySet(models.QuerySet):
    pass


class AssociationSpecificDataManager(models.Manager):
    def get_queryset(self):
        return AssociationSpecificDataQuerySet(self.model, using=self._db)


class AssociationSpecificData(models.Model):
    """
    This model stores a value for a data field that belongs to an association, e.g. if an association has a specific
    data field called 'Drivers licence' then this model stores whether the user linked to the membership has a
    'Drivers licence'.
    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    last_modified = models.DateTimeField(auto_now=True)
    value = models.TextField(blank=True)

    association = models.ForeignKey(Association, related_name="specific_data", on_delete=models.CASCADE)
    membership = models.ForeignKey(AssociationMembership, related_name="specific_data", on_delete=models.CASCADE)
    data_field = models.ForeignKey(AssociationSpecificDataFields, related_name="specific_data", on_delete=models.CASCADE)

    objects = AssociationSpecificDataManager()

    class Meta:
        verbose_name_plural = "AssociationSpecificData"
        unique_together = ('membership', 'data_field')

    def __str__(self):
        return str(self.data_field) + str(self.membership)

    def clean(self):
        if self.association != self.membership.association:
            raise ValidationError("Association of membership doesn't match the association.")
        if self.association != self.data_field.association:
            raise ValidationError("Association of data_field doesn't match the association.")
        if self.data_field.type == "Boolean" and self.data_field.mandatory and not self.value:
            raise ValidationError("Value has to be true, when data field is mandatory.")

        if self.data_field_id:
            if self.association != self.data_field.association:
                raise ValidationError({"data_field": "Association of data_field doesn't match the association."})

            # TODO: remove after django update to v3
            if len(self.data_field.type[0]) > 1:
                print(len(self.data_field.type))
                print(self.data_field.type[0])
                raise ValidationError({"data_field": "Validation does not work currently for more than one type"})

            if self.data_field.type == "Boolean":
                if self.value not in ["true", "false"]:
                    raise ValidationError({"value": "Value for boolean data field should be 'true' or 'false'."})
                if self.data_field.mandatory and self.value != "true":
                    raise ValidationError({"value": "Value has to be true (value='true'), when data field is mandatory."})
            elif self.data_field.type == "Number":
                if not self.value.isdigit():
                    raise ValidationError({"value": "Value should be a integer"})
            elif self.data_field.type == "String":
                # any valid value for a text field is valid for a data field with type string
                pass
            elif self.data_field.type == "Choice":
                if self.value not in self.data_field.choices.split(","):
                    raise ValidationError({"value": "Value {} is not in the list of valid choices"
                                          .format(self.value)})
            else:
                # Catch all safeguard to prevent rogue data in the data base that needs to be fixed manually when
                # validation is forgotten to be implemented.
                raise ValidationError({"value": "Validation for {} type of data field has not been implemented"
                                      .format(self.data_field.type)})
