import uuid

from django.core.exceptions import ValidationError
from django.db import models

from apps.MySU.models import Association


class AssociationSpecificDataFieldsQuerySet(models.QuerySet):
    def mandatory(self):
        return self.filter(mandatory=True)


class AssociationSpecificDataFieldsManager(models.Manager):
    def get_queryset(self):
        return AssociationSpecificDataFieldsQuerySet(self.model, using=self._db)

    def mandatory(self, association):
        return self.get_queryset().filter(association=association).mandatory()


class AssociationSpecificDataFields(models.Model):
    """
    This model represents a generic custom field for an association. It is infeasible to add all the possible
    data fields every association would need to the membership model. Hence every association can make generic fields
    specific for their association.

    Note that this might mean that multiple association will have the same specific data field, but this minor overhead/
    inconvenience is preferred over the infeasibility of adding fields for every association to the membership.

    # type
    There are 4 type choices for the type. This does not change how the data is stored in the database; it will be a
    string regardless. It will change how it will be validated though. Having this type option, means that association
    can distinguish whether they expect a yes/no or a random word.

    # helper_text
    This stores a description about the specific data field.

    # choices
    When choosing the type "Choice", the list of valid choices needs to be stored somewhere. This is done in this field
    as a comma separated list.

     # default
     When this field is created, it is likely that the association has members. It could be that a blank value is fine,
     but to be sure every member gets a data field containing the value set by default.

     # include_with_members_incoming
     Currently only name and email are listed for incoming members on the frontend. This is a field that could indicate
     that this should be included in that list.

     # mandatory
     A boolean type can be made mandatory, i.e. when the boolean type has mandotory=true it must be true to pass
     validation.

    """
    class Type(models.TextChoices):
        BOOLEAN = "Boolean", "Boolean"
        NUMBER = "Number", "Number"
        STRING = "String", "String"
        CHOICE = "Choice", "Choice"
        COUNTRY = "Country", "Country"

    slug = models.SlugField(default=uuid.uuid4, editable=False)
    association = models.ForeignKey(Association, on_delete=models.CASCADE)
    type = models.CharField(max_length=20, choices=Type.choices, default=Type.BOOLEAN)
    name = models.CharField(max_length=50, blank=False)
    helper_text = models.TextField(max_length=200, blank=True, default='')
    choices = models.CharField(max_length=200, null=True, blank=True)
    default = models.TextField(blank=True)
    include_with_members_incoming = models.BooleanField(default=False)
    board_only = models.BooleanField(default=False)
    mandatory = models.BooleanField(default=False)

    objects = AssociationSpecificDataFieldsManager()

    class Meta:
        verbose_name_plural = "AssociationSpecificDataFields"
        unique_together = ('name', 'association')

    def __str__(self):
        return str(self.name)

    def clean(self):
        if self.type == 'Choice':
            if not isinstance(self.choices, str):
                raise ValidationError("Expected `choices` to be a string.")
            if len(self.choices.split(',')) < 1:
                raise ValidationError("Expected `choices` to have at least a choice")
            if ", " in self.choices:
                raise ValidationError("Choices need to be comma separated without spaces behind the comma")

        if self.default is not None:
            if self.type == "Number":
                if not (self.default.isdigit()):
                    raise ValidationError("Default {} should be an integer, but it is not.".format(self.default))
            elif self.type == "String":
                if not isinstance(self.default, str):
                    raise ValidationError("Default {} should be a string, but it is not.".format(self.default))
            elif self.type == "Choice":
                if self.default not in self.choices.split(','):
                    raise ValidationError("Default {} is not a valid choice.".format(self.default))

        if self.mandatory:
            if self.type != "Boolean":
                raise ValidationError("Only Boolean fields can be mandatory.")
            if self.default != "False":
                raise ValidationError("Mandatory fields must have a 'No' (value='False') default.")
            if self.board_only:
                raise ValidationError("Field cannot be mandatory and be board only.")
