import datetime
import uuid

from computedfields.models import computed
from computedfields.models import ComputedFieldsModel
from django.contrib.auth.models import Group as DjangoGroup
from django.core.exceptions import ValidationError
from django.db import models
from guardian.shortcuts import get_objects_for_group


class MemberGroupQuerySet(models.QuerySet):
    pass


class MemberGroupManager(models.Manager):
    def get_queryset(self):
        return MemberGroupQuerySet(self.model, using=self._db)


class Group(ComputedFieldsModel, DjangoGroup):
    """
    Members of an association can be put into committee, workgroups, study groups, clubs, etc. These are all a instances
    of a group. The django Group is a collection of users. However, the default Django group is limited to just a unique
    name. We could have created a Group that doesn't extend the Django Group, however the Django permission system is
    based around this Group and they do not allow a drop-in replacement for the default django group. This means that we
    need to extend the django group. As a result we are stuck with the name field on the django group. This name is
    unique, and because it is such just pretending it doesn't exist does not work. To work around this the name will be
    populated with a uuid version 4.
    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    full_name = models.CharField(max_length=100)
    short_name = models.CharField(max_length=100, blank=True, null=True)
    alternate_name = models.CharField(max_length=100, blank=True, null=True)
    number = models.PositiveSmallIntegerField(blank=True, null=True)
    creed = models.CharField(max_length=200, blank=True, null=True)
    theme = models.CharField(max_length=200, blank=True, null=True)
    edition = models.CharField(max_length=200, blank=True, null=True)
    start_year = models.PositiveSmallIntegerField(blank=True, null=True)
    end_year = models.PositiveSmallIntegerField(blank=True, null=True)
    founding_date = models.DateField(default=datetime.date.today)
    dissolution_date = models.DateField(blank=True, null=True)
    archived = models.BooleanField(default=False)
    description = models.TextField(blank=True, null=True)
    photo = models.ImageField(upload_to='group/photo/', null=True, blank=True)
    logo = models.ImageField(upload_to='group/logo/', null=True, blank=True)
    board_group = models.BooleanField(default=False)
    email = models.EmailField(null=True, blank=True)

    django_group = models.OneToOneField(DjangoGroup, parent_link=True, related_name='group', on_delete=models.CASCADE)
    association = models.ForeignKey('MySU.Association', related_name="groups", on_delete=models.PROTECT)
    parent_group = models.ForeignKey('self', null=True, blank=True, on_delete=models.CASCADE)

    @computed(models.BooleanField(default=False), depends=[['groupobjectpermission_set', ['id']], ['association', ['id']]])
    def current_board(self):
        return self.association in get_objects_for_group(self, ["MySU.board"], accept_global_perms=False)

    objects = MemberGroupManager()

    class Meta:
        verbose_name = "Group"
        verbose_name_plural = "Groups"

    def __str__(self):
        return "{association} - {name}".format(association=self.association_id, name=self.full_name)

    def clean(self):
        if self.dissolution_date:
            if self.dissolution_date < self.founding_date:
                raise ValidationError("Dissolution must occur after founding")
        if self.parent_group_id:
            if self.parent_group.association != self.association:  # pylint: disable=no-member
                raise ValidationError("Group must be part of the same association as its parent")

    def save(self, *args, **kwargs):
        is_new = not self.pk
        if is_new:
            self.name = str(uuid.uuid4())
        super().save(*args, **kwargs)
