import datetime
import uuid

from django.contrib.auth.models import Group as DjangoGroup
from django.core.exceptions import ValidationError
from django.db import models
from simple_history.models import HistoricalRecords


class GroupMembershipQuerySet(models.QuerySet):
    def current(self):
        return self.filter(date_left__lte=datetime.date.today())

    def for_association(self, association: 'association.Association'):
        return self.filter(association=association)

    def for_user(self, user: 'user.User'):
        return self.filter(user=user)


class GroupMembershipManager(models.Manager):
    def get_queryset(self):
        return GroupMembershipQuerySet(self.model, using=self._db)

    def for_user(self, user: 'user.User'):
        return self.get_queryset().for_user(user)

    def for_association(self, association: 'association.Association'):
        return self.get_queryset().for_association(association)

    def for_user_and_association(self, user: 'user.User', association: 'association.Association'):
        return self.get_queryset().for_user(user).for_association(association)

    def current(self, user=None, association=None):
        queryset = self.get_queryset().current()
        if user:
            if association is not None:
                queryset = queryset.for_user_and_association(user, association)
            else:
                queryset = queryset.for_user(user)
        elif association:
            queryset = queryset.for_association(association)
        return queryset

    def emails(self, user=None, association=None):
        if user is None:
            return self.only('email')
        else:
            return self.for_user_and_association(user, association).only('email')


class GroupMembership(models.Model):
    """
    This model links the member to a group.

    # duty
    Description of what role the member fulfills within the group

    # description (only used by board)
    Some text telling something related to the member.

    # archived
    Currently has no function

    # email
    This is the alias for the group. When an email is sent to this alias, every member of this group will get an email.

    # order
    There are places that the ordering of member might be a little more complex than alphabetically. This value can be
    used as a way to order the users in a desired way.
    """
    slug = models.UUIDField(default=uuid.uuid4, editable=False)
    duty = models.CharField(max_length=100, default="General member", blank=False)
    description = models.TextField(default="", blank=True)
    date_joined = models.DateField(default=datetime.date.today)
    date_left = models.DateField(null=True, blank=True)
    archived = models.BooleanField(default=False)
    email = models.EmailField(null=True, blank=True)
    order = models.PositiveSmallIntegerField(default=1)

    profile = models.ForeignKey('MySU.Profile', related_name='group_member', on_delete=models.PROTECT)
    user = models.ForeignKey('MySU.User', on_delete=models.PROTECT, null=True)
    django_group = models.ForeignKey(DjangoGroup, related_name='groupmemberships', on_delete=models.CASCADE)
    association = models.ForeignKey('MySU.Association', related_name='groupmemberships', on_delete=models.PROTECT)

    # history = HistoricalRecords()

    objects = GroupMembershipManager()

    class Meta:
        verbose_name = "Group Membership"
        verbose_name_plural = "Group Memberships"
        constraints = [
            models.UniqueConstraint(fields=['profile', 'django_group', 'date_left'],
                                    name="group_memberships_unique_with_date_left"),
            models.UniqueConstraint(fields=['profile', 'django_group'],
                                    condition=models.Q(date_left=None),
                                    name="group_memberships_unique_without_date_left"),
        ]

    def __str__(self):
        return "{user} of {group}".format(user=self.user, group=self.django_group)

    def clean(self):
        if self.user_id:
            if self.user != self.profile.user:
                raise ValidationError("Profile must belong to the user")
        if self.date_left:
            if self.date_left < self.date_joined:
                raise ValidationError(
                    {'date_left': "End date must be after start date."},
                )
