import uuid

from django.core.validators import MinValueValidator
from django.db import models

from apps.MySU.models import Association
from MySU.fields.yearlessdate import YearlessDateField
from MySU.objects import YearlessDate


class MemberTypeQuerySet(models.QuerySet):
    pass


class MemberTypeManager(models.Manager):
    def get_queryset(self):
        return MemberTypeQuerySet(self.model, using=self._db)


class MemberType(models.Model):
    """
    Every member needs to have a type. The member types an association can have are stored in MemberTypes models.

    # type
    Name of the member type, e.g. General, Alumni or Honorary

    # disabled
    To preserve history not every member_type is removed on a delete call. This is done to preserve history, in particular
    to preserve the membership fees. When set to true, this cannot be assigned to new membership.

    # only_chooseable_by_board
    Some membertypes should not be able to be chosen by anyone. A classic example is the Honorary member. This is
    awarded not chosen, hence some types should only be able to be chosen by the board.

    # full_year_membership
    This is a membership that runs for the full length of the year

    # auto_end
    Some memberships are for a single year only, and should end afterwards. Setting this field to True, means that every
    member with this member type will automatically end their membership at the end of the year. Setting it to False,
    means that the membership is continuous.

    # start_date & end_date
    Membership can be limited to parts of the year, e.g. only for the summer. These field can be used to specify that.

    # membership_fee
    Every membership can have a different cost. This cost is the membership fee

    # payment_per
    It is possible that payments are done on a monthly, semesterly or yearly basis. Currently no functionality uses
    this option.

    # grace_period
    Consider the situation that an association has two member types: a full year membership and a half year membership.
    Suppose someone enrolls with a half year membership. If that person whats to change their membership, then this can
    be done within the grace_period.

    """
    class PaymentPer(models.TextChoices):
        YEAR = "Year", "Year"
        SEMESTER = "Semester", "Semester"
        MONTH = "Month", "Month"
        WEEK = "Week", "Week"

    slug = models.SlugField(default=uuid.uuid4, editable=False)
    type = models.CharField(max_length=200)
    disabled = models.BooleanField(default=False)
    only_chooseable_by_board = models.BooleanField(default=False)
    order = models.PositiveSmallIntegerField(default=1)
    helper_text = models.TextField(max_length=200, blank=True, default='')
    full_year_membership = models.BooleanField(default=True)
    auto_end = models.BooleanField(default=False)
    start_date = YearlessDateField(default=YearlessDate(9, 1))
    end_date = YearlessDateField(default=YearlessDate(8, 31))
    membership_fee = models.DecimalField(validators=[MinValueValidator(0.0)], decimal_places=2, max_digits=5)
    payment_per = models.CharField(max_length=20, choices=PaymentPer.choices, default=PaymentPer.YEAR)
    grace_period = models.PositiveSmallIntegerField(default=7)

    association = models.ForeignKey(Association, on_delete=models.PROTECT)

    objects = MemberTypeManager()

    def __str__(self):
        return "{association} - {type}".format(type=self.type, association=self.association_id)
