import datetime
import uuid

from computedfields.models import computed
from computedfields.models import ComputedFieldsModel
from django.contrib.auth.base_user import AbstractBaseUser
from django.contrib.auth.models import Group as DjangoGroup
from django.contrib.auth.models import PermissionsMixin
from django.contrib.auth.models import UserManager as DjangoUserManager
from django.core.exceptions import ValidationError
from django.core.mail import send_mail
from django.db import models
from django.utils.translation import ugettext_lazy as _
from guardian.mixins import GuardianUserMixin

from apps.MySU.models import Profile
from MySU.validators.validators import VALIDATE_RECOVERY_EMAIL


class UserQuerySet(models.QuerySet):
    pass


class UserManager(DjangoUserManager):
    def get_queryset(self):
        return UserQuerySet(self.model, using=self._db)


class User(ComputedFieldsModel, AbstractBaseUser, PermissionsMixin, GuardianUserMixin):
    """
    This is the User. ...
    """
    slug = models.UUIDField(default=uuid.uuid4, editable=False)
    username = models.CharField(max_length=255, unique=True)
    student_number = models.CharField(max_length=100, blank=True, null=True)
    surfconext_signin = models.BooleanField(default=False)
    date_joined = models.DateField(default=datetime.date.today)
    date_activated = models.DateField(blank=True, null=True)
    date_left = models.DateField(blank=True, null=True)
    is_active = models.BooleanField(default=True)
    is_staff = models.BooleanField(default=False)
    is_api = models.BooleanField(default=False)
    email = models.EmailField(validators=[VALIDATE_RECOVERY_EMAIL], blank=False, null=True)

    preferred_profile = models.OneToOneField(Profile, related_name="+", null=True, on_delete=models.SET_NULL, blank=True)
    # studies = models.ManyToManyField('MySU.Study', through='StudyRecord', related_name='users')
    # Added this to add a through model; must be called groups for other packages to work
    groups = models.ManyToManyField(
        DjangoGroup,
        verbose_name=_('groups'),
        blank=True,
        help_text=_(
            'The groups this user belongs to. A user will get all permissions '
            'granted to each of their groups.'
        ),
        related_name="user_set",
        related_query_name="user",
        through='MySU.GroupMembership'
    )

    # history = HistoricalRecords()

    @computed(models.PositiveSmallIntegerField(default=0), depends=[['profiles', ['id']]])
    def profile_count(self):
        return self.profiles.count()

    objects = DjangoUserManager()

    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email']
    EMAIL_FIELD = 'email'

    class Meta:
        verbose_name = "User"
        verbose_name_plural = "Users"

    def __str__(self):
        return self.username

    def clean(self):
        if self.date_left:
            if self.date_left < self.date_activated:
                raise ValidationError("Deactivation must occur after founding")
        if self.preferred_profile_id:
            if self.preferred_profile.user != self:
                raise ValidationError("A User can only prefer a Profile belonging to the user")

    def get_full_name(self):
        return self.username

    def get_short_name(self):
        return self.username

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Send an email to this user."""
        send_mail(subject, message, from_email, [self.email], **kwargs)
