from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from apps.MySU.models import AssociationMembership
from apps.MySU.models import AssociationSpecificData
from apps.MySU.models import AssociationSpecificDataFields
from apps.MySU.serializers.base import BaseSerializer
from MySU.fields.association import AssociationField
from MySU.helper_functions import get_for_patch


class AssociationSpecificDataFieldsSerializer(BaseSerializer):
    type = serializers.CharField()
    association = AssociationField(view_name='association-detail', lookup_field='slug')

    def validate(self, attrs):
        attrs = super().validate(attrs)
        instance = self.instance

        type_ = get_for_patch(instance, attrs, 'type')
        choices = get_for_patch(instance, attrs, 'choices')
        mandatory = get_for_patch(instance, attrs, 'mandatory')
        default = get_for_patch(instance, attrs, 'default')
        board_only = get_for_patch(instance, attrs, 'board_only')

        if type_ == 'Choice':
            if not isinstance(choices, str):
                raise ValidationError("Expected `choices` to be a string.")
            if len(choices.split(',')) < 1:
                raise ValidationError("Expected `choices` to have at least a choice")
        if type_ not in AssociationSpecificDataFields.Type.values:
        # if type_ not in ["Boolean", "Choice", "Number", "String"]:
            raise ValidationError({"type": "Improper type given"})

        if default is not None:
            if type_ == "Number":
                if not default.isdigit():
                    raise ValidationError("Default {} should be an integer encapsulated within string, but it is not.".format(default))
            elif type_ == "String":
                if not isinstance(default, str):
                    raise ValidationError("Default {} should be a string, but it is not.".format(default))
            elif type_ == "Choice":
                if default not in choices.split(',') and default != "":
                    raise ValidationError("Default {} is not a valid choice.".format(default))
            elif type_ == "Boolean":
                if default not in ["True", "False"]:
                    raise ValidationError("Default {} has should be a capitalised boolean, i.e. True or False, encapsulated within string, but it is not.".format(default))
                attrs["default"] = default == "True"

        if mandatory:
            if type_ != "Boolean":
                raise ValidationError("Only Boolean fields can be mandatory.")
            if board_only:
                raise ValidationError("Field cannot be mandatory and be board only.")
            if default != "False":
                raise ValidationError("Mandatory fields must have a 'No' (value='False') default.")

        return attrs

    def create(self, validated_data):
        data_field = super(AssociationSpecificDataFieldsSerializer, self).create(validated_data)
        association = validated_data["association"]
        to_be_updated_memberships = AssociationMembership.objects.filter(association=association)
        to_be_created_specific_data = []
        for membership in to_be_updated_memberships:
            specific_data = AssociationSpecificData(
                association=association,
                membership=membership,
                data_field=data_field,
                value=data_field.default
            )
            to_be_created_specific_data.append(specific_data)
        AssociationSpecificData.objects.bulk_create(objs=to_be_created_specific_data, batch_size=1000)
        return data_field

    class Meta:
        model = AssociationSpecificDataFields
        fields = ('url', 'slug', 'association', 'name', 'type', 'choices', 'board_only', 'mandatory', 'default', 'helper_text')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
        }
        create_only_fields = ('type', 'choices')
