from rest_framework import serializers


class BaseSerializer(serializers.HyperlinkedModelSerializer):
    """
    The serializer underlying every other serializer in the project.

    This serializer expands upon the HyperlinkedModelSerializer. This adds an url to every endpoint based on a field in
    order to facilitate easy access to related models. The expansion made are to cover shortcommings of the
    HyperlinkedModelSerializer.

    The first change is based on the package ""drf-queryfields". It allows passing an include and exclude argument that
    allows for settings fields the serializer passed. The usage of the include and exclude arguments can be understood
    based on the example of a model with fields "id, text, title, association". A query with
    ?<include_arg_name>=text,title gives a result with {text: ..., title: ...}, and a query with
    ?<exclude_arg_name>=text,title gives a result with {id: ..., association: ...}.

    The next change is "nested_fields". When a serializer gets nested=True, all fields in the nested_fields argument
    will be omitted. This is useful for not double representing related object in a nested serializer.

    The third change is the addition of "create_only_fields". Django provides read_only, write_only and fields that can
    be set on some default. This default is however not flexible. To make it flexible we added "create_only_fields".
    These fields are included with the POST request, but set to read_only for PATCH/PUT.

    The fourth and final change is the addition of "board_only_fields" and "user_only_fields" only fields. These field
    specify what fields boards are allowed to see additionally to the normal user, and what fields the associated user
    is allowed to see additionally to the normal user. This prevents needlessly many serializer for each edge case in
    which the user or board member is shown some more information than the normal member of an association.
    """

    # If using Django filters in the API, these labels mustn't conflict with any model field names.
    include_arg_name = 'fields'
    exclude_arg_name = 'fields!'

    # Split field names by this string.  It doesn't necessarily have to be a single character.
    # Avoid RFC 1738 reserved characters i.e. ';', '/', '?', ':', '@', '=' and '&'
    delimiter = ','

    def __init__(self, *args, **kwargs):
        nested = kwargs.pop('nested', None)
        super(BaseSerializer, self).__init__(*args, **kwargs)

        if nested:
            for field in getattr(self.Meta, 'nested_fields', []):
                self.fields.pop(field, None)

    def get_field_names(self, declared_fields, info):
        expanded_fields = super(BaseSerializer, self).get_field_names(declared_fields, info)

        if getattr(self.Meta, 'extra_fields', None):
            expanded_fields += self.Meta.extra_fields

        request = self.context.get('request', None)
        if request:
            query_params = getattr(request, 'query_params', request.GET)

            includes = query_params.getlist(self.include_arg_name)
            include_field_names = {name for names in includes for name in names.split(self.delimiter) if name}

            excludes = query_params.getlist(self.exclude_arg_name)
            exclude_field_names = {name for names in excludes for name in names.split(self.delimiter) if name}

            serializer_field_names = set(expanded_fields)

            fields_to_drop = serializer_field_names & exclude_field_names
            if include_field_names:
                fields_to_drop |= serializer_field_names - include_field_names

            return tuple(serializer_field_names - fields_to_drop)
        return tuple(expanded_fields)

    def to_representation(self, instance):
        # Get the original representation
        representation = super(BaseSerializer, self).to_representation(instance)

        # Remove create_only_fields when not creating
        create_only_fields = getattr(self.Meta, 'create_only_fields', [])
        user_only_fields = getattr(self.Meta, 'user_only_fields', [])
        board_only_fields = getattr(self.Meta, 'board_only_fields', [])

        request = self.context.get('request', None)
        if request is not None:
            if create_only_fields and request.method in ["PATCH", "PUT"]:
                for field in create_only_fields:
                    representation.pop(field, None)
            if request.user:
                if user_only_fields:
                    if instance.user_id != request.user.pk:
                        for field_dict in user_only_fields:
                            if not (field_dict['read'] and request.method in ["GET", "LIST"]):
                                representation.pop(field_dict['field'], None)
                if board_only_fields and not request.user.has_perm('board', instance.association):
                    for field_dict in board_only_fields:
                        if not (field_dict['read'] and request.method in ["GET", "LIST"]):
                            representation.pop(field_dict['field'], None)
        return representation

    class Meta:
        extra_fields = None
        board_only_fields = []
        user_only_fields = []
        create_only_fields = []
        nested_fields = []
