import datetime

from django.contrib.auth.models import ContentType
from guardian.models import GroupObjectPermission
from rest_framework import serializers

from apps.MySU.models import Group
from apps.MySU.models import SUNPermission
from apps.MySU.serializers.base import BaseSerializer
from apps.MySU.serializers.group_membership import GroupMembershipSerializer
from MySU.fields.association import AssociationField
from MySU.fields.permission import PermissionField


class GroupSerializer(BaseSerializer):
    days_in_existence = serializers.SerializerMethodField()
    association = AssociationField(view_name='association-detail', lookup_field='slug')
    permissions = PermissionField(source='groupobjectpermission_set', slug_field='codename', many=True,
                                  queryset=SUNPermission.objects.all(), required=False)
    groupmemberships = GroupMembershipSerializer(many=True, read_only=True, nested=True)

    @staticmethod
    def get_days_in_existence(obj):
        return (datetime.date.today() - obj.founding_date).days

    class Meta:
        model = Group
        exclude = ('name',)
        nested_fields = ['association', 'permissions']
        create_only_fields = ['association', ]
        extra_kwargs = {
            'parent_group': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'url': {'lookup_field': 'slug'},
            'groupmemberships': {'lookup_field': 'slug',
                                 'read_only': 'read_only'},
            "permissions": {'required': False}
        }
        extra_fields = ('groupmemberships',)

    def validate(self, attrs):
        groupobjectpermissions = attrs.pop('groupobjectpermission_set', None)
        attrs = super().validate(attrs)
        if groupobjectpermissions is not None:
            for perm in groupobjectpermissions:
                perm.clean()
            attrs.update({'groupobjectpermission_set': groupobjectpermissions})
        return attrs

    def create(self, validated_data):
        # Remove the groupobjectpermissions from the attrs
        groupobjectpermissions = validated_data.pop('groupobjectpermission_set', None)
        # Create the group as usual
        group = Group(**validated_data)
        group.save()

        # Handle the groupobjectpermissions
        association = validated_data['association']
        content_type = ContentType.objects.get_for_model(association)
        if groupobjectpermissions is not None:
            GroupObjectPermission.objects.bulk_create(
                [GroupObjectPermission(
                    permission=perm,
                    content_type=content_type,
                    object_pk=association.id,
                    group=group.django_group
                ) for perm in groupobjectpermissions]
            )
        return group

    def update(self, instance: Group, validated_data):
        groupobjectpermissions = validated_data.pop('groupobjectpermission_set', None)
        if groupobjectpermissions is not None:
            # just delete all and then recreate is easier to code.
            GroupObjectPermission.objects.filter(group=instance.django_group).delete()
            content_type = ContentType.objects.get(model='association')
            GroupObjectPermission.objects.bulk_create(
                [GroupObjectPermission(
                    permission=perm,
                    content_type=content_type,
                    object_pk=instance.association.id,
                    group=instance.django_group
                ) for perm in groupobjectpermissions]
            )
        return super().update(instance, validated_data)
