from django.core.mail import send_mail
from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from apps.MySU.models import AssociationMembership
from apps.MySU.models import Group
from apps.MySU.models import GroupMembership
from apps.MySU.models import Profile
from apps.MySU.serializers.base import BaseSerializer
from apps.MySU.serializers.profile import ProfileSerializer
from MySU.fields.presentable_hyperlinked_related import PresentableHyperlinkedRelatedField
from MySU.helper_functions import get_for_patch


class GroupMembershipSerializerList(BaseSerializer):
    group = serializers.HyperlinkedRelatedField(
        source='django_group.group', lookup_field='slug',
        view_name='group-detail', queryset=Group.objects.all()
    )
    full_name = serializers.ReadOnlyField(source="profile.full_name")

    class Meta:
        model = GroupMembership
        exclude = ('django_group', 'user')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'profile': {'lookup_field': 'slug'},
        }


class GroupMembershipSerializer(BaseSerializer):
    group = serializers.HyperlinkedRelatedField(
        source='django_group.group', lookup_field='slug',
        view_name='group-detail', queryset=Group.objects.all()
    )
    full_name = serializers.ReadOnlyField(source="profile.full_name")

    class Meta:
        model = GroupMembership
        exclude = ('django_group', 'user')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'profile': {'lookup_field': 'slug'},
            'group': {'lookup_field': 'slug'},
        }
        create_only_fields = ('association', 'profile')
        nested_fields = ['association', 'group']

    def validate(self, attrs):
        instance = self.instance

        association = get_for_patch(instance, attrs, 'association')

        user = self.context['request'].user
        if not user.has_perm("board", association):
            raise ValidationError({"association": "User must be board in order to change or create a groupmembership."})

        # On creation:
        if instance is None:
            # Set User equal to User of supplied profile
            profile = attrs['profile']
            attrs['user'] = profile.user

            # Check if supplied profile is member of the given association
            if not AssociationMembership.objects.filter(association=attrs['association'], profile=profile).exists():
                raise ValidationError("Supplied member {member} not a member of supplied association {association}"
                                      .format(member=profile.slug, association=attrs['association'].slug))

            # DRF is smart enough that when supplied with a group it sets django_group to {'group': <Group ...>}
            # We only want to pass a django group not a dict, so we get the django group from the information in the dict.
            attrs['django_group'] = attrs['django_group']["group"].django_group
        return attrs

    def create(self, validated_data):
        group_membership = super().create(validated_data)
        group = group_membership.django_group.group
        send_mail(
            f"Added to {group.short_name}",
            f"Hi {group_membership.profile.given_name}\n\n"
            f"You have been added to {group.full_name} of {group_membership.association.short_name}.\n\n"
            f"If you have any further question, please contact the board.\n\n"
            f"Kind regards,\n\n"
            f"The MySU team",
            "no-reply@su.utwente.nl",
            [group_membership.profile.email]
        )
        return group_membership
