from rest_framework.exceptions import ValidationError

from apps.financial.models import MembershipFee
from apps.MySU.models import AssociationMembership
from apps.MySU.models import MemberType
from apps.MySU.serializers.base import BaseSerializer
from MySU.fields.association import AssociationField
from MySU.helper_functions import get_for_patch


class MemberTypeSerializer(BaseSerializer):
    association = AssociationField(view_name='association-detail', lookup_field='slug')

    class Meta:
        model = MemberType
        fields = ('url', 'slug', 'type', 'association', 'only_chooseable_by_board', 'start_date', 'end_date', 'membership_fee',
                  'order', 'helper_text', 'full_year_membership', 'disabled')
        create_only_fields = ('association',)
        nested_fields = ['association']
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'}
        }

    def validate(self, attrs):
        attrs = super().validate(attrs)
        slug = get_for_patch(self.instance, attrs, 'slug')
        type_name = get_for_patch(self.instance, attrs, 'type')
        association = get_for_patch(self.instance, attrs, 'association')
        active_association_member_types = MemberType.objects.filter(association=association, disabled=False).exclude(slug=slug)

        errors = {}
        if type_name in [member_type.type for member_type in active_association_member_types]:
            errors["type"] = f"No two active member types can have the same name."
        if errors:
            raise ValidationError(errors)

        return attrs

    def update(self, instance: MemberType, validated_data):
        has_fees_associated_to_it = MembershipFee.objects.filter(membertype=instance).exists()
        if has_fees_associated_to_it:
            # Create new member type.
            new_member_type = MemberType(**validated_data)
            new_member_type.save()
            # Make sure the older member type can no longer be used.
            instance.disabled = True
            instance.save()
            # Update the current memberships to use the newly created member type instead of the old one.
            AssociationMembership.objects.filter(type=instance).update(type=new_member_type)
            AssociationMembership.objects.filter(new_type=instance).update(new_type=new_member_type)
            # Return member type, just as if it was normally updated.
            return new_member_type
        else:
            return super().update(instance, validated_data)
