from django.core.mail import send_mail
from rest_flex_fields import FlexFieldsModelSerializer
from rest_framework import serializers

from apps.financial.models import BankAccount
from apps.financial.models import DebtEntry
from apps.MySU.models import Profile
from apps.MySU.serializers.base import BaseSerializer
from apps.MySU.serializers.study import StudySerializer


class ProfileSerializerList(BaseSerializer):
    class Meta:
        depth = 0
        model = Profile
        fields = ('url', 'profilename')
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
        }


class ProfileSerializer(FlexFieldsModelSerializer, serializers.HyperlinkedModelSerializer):
    bank_account_name = serializers.CharField(source="bank_account.name", required=False, allow_blank=True)
    iban = serializers.CharField(source="bank_account.iban", required=False, allow_blank=True)
    bic = serializers.CharField(source="bank_account.bic", required=False, allow_blank=True)
    mine = serializers.SerializerMethodField()
    """
    The class that allows board members to view of the User
    """

    def get_mine(self, obj):
        user = self.context['request'].user
        return obj.user_id == user.id

    class Meta:
        model = Profile
        exclude = ('user', 'bank_account')
        lookup_field = 'slug'
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'student': {'lookup_field': 'slug'},
            'study': {'lookup_field': 'slug'},
            'user': {'lookup_field': 'slug',
                     'read_only': True},
            'student_number': {'read_only': True}
        }
        expandable_fields = {
            'study': StudySerializer
        }

    def update(self, instance: Profile, validated_data):
        bank_account = validated_data.pop("bank_account", None)
        if bank_account:
            bank_account_name = bank_account.get("name", None)
            iban = bank_account.get("iban", None)
            bic = bank_account.get("bic", None)
        else:
            bank_account_name = iban = bic = None

        if iban is not None or bic is not None or bank_account_name is not None:
            bank_account = instance.bank_account
            if DebtEntry.objects.filter(bank_account=bank_account, debt_collection__locked=True).exists():
                if iban is None:
                    iban = bank_account.iban
                if bic is None:
                    bic = bank_account.bic
                if bank_account_name is None:
                    bank_account_name = bank_account.name
                bank_account = BankAccount.objects.create(iban=iban, bic=bic, name=bank_account_name)
                instance.bank_account = bank_account
            else:
                if iban is not None:
                    bank_account.iban = iban
                if bic is not None:
                    bank_account.bic = bic
                if bank_account_name is not None:
                    bank_account.name = bank_account_name
                bank_account.save()
        return super().update(instance, validated_data)

    def create(self, validated_data):
        bank_account = validated_data.pop("bank_account", None)
        if bank_account:
            bank_account_name = bank_account.get("name", "")
            iban = bank_account.get("iban", "")
            bic = bank_account.get("bic", "")
            bank_account = BankAccount.objects.create(iban=iban, bic=bic, name=bank_account_name)
            validated_data['bank_account'] = bank_account

        user = self.context['view'].request.user
        validated_data['user'] = user
        validated_data['student_number'] = user.student_number
        profile = super(ProfileSerializer, self).create(validated_data)
        if not user.preferred_profile_id:
            send_mail(
                "Welcome",
                f"Hi {validated_data['given_name']}\n\nThank you for creating an account. Welcome to the platform!\n\nKind regards,\n\nThe MySU team",
                "no-reply@su.utwente.nl",
                [validated_data["email"]]
            )
            user.preferred_profile = profile
            user.save()
        return profile
