import json
import uuid
from json.decoder import JSONDecodeError

from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from rest_framework import permissions
from rest_framework.views import APIView

from apps.MySU.models import AssociationMembership


# TODO: Rewrite this using status codes, so that frontend knows when the call failed verification
@method_decorator(csrf_exempt, name='dispatch')
class MembershipResolveView(APIView):
    http_method_names = ['post']
    permission_classes = (permissions.IsAuthenticated,)

    def __init__(self):
        super(MembershipResolveView, self).__init__()
        self.action_to_status_dict = {
            'accept': AssociationMembership.Status.ACCEPTED,
            'Accepted': AssociationMembership.Status.ACCEPTED,
            'confirm': AssociationMembership.Status.ACCEPTED,
            'dispute': AssociationMembership.Status.DISPUTED,
            'Disputed': AssociationMembership.Status.DISPUTED
        }

    def post(self, request):
        user = request.user

        # Check whether proper json
        try:
            membership_dict = request.data
        except JSONDecodeError as e:
            return JsonResponse({
                "code": "MR1",
                "reason": "Input must be a dict of the form {{slug: action, slug: action, ...}} "
                          "with slug a valid v4 uuid and action in {}. Malformed Json given: {} with error {}"
                          .format(self.action_to_status_dict.keys(), request.body.decode('utf-8'), e)
            }, status=400)

        # Check whether proper format
        if membership_dict is None or not self.has_proper_data_format(membership_dict):
            return JsonResponse({
                "code": "MR2",
                "reason": "Input must be a dict of the form {{slug: action, slug: action, ...}} "
                          "with slug a valid v4 uuid and action in {}. Improper form.".format(self.action_to_status_dict.keys()),
                "dict": "{}".format(membership_dict)
            }, status=400)

        # Check whether number of claimed memberships in database matches the number of given memberships
        found_memberships = AssociationMembership.objects.filter(user=user, status=AssociationMembership.Status.CLAIMED)
        number_of_found_memberships = len(found_memberships)
        number_of_given_memberships = len(membership_dict)
        if number_of_found_memberships != number_of_given_memberships:
            return JsonResponse({
                'code': 'MR3',
                'reason': "Number of claimed membership found is {}, {} given. Please post only and all claimed "
                          "memberships.".format(number_of_found_memberships, number_of_given_memberships)
            }, status=400)

        # Check whether claimed memberships in database matches the given memberships
        slugs_of_found_memberships = {str(membership.slug) for membership in found_memberships}
        slugs_of_given_memberships = {membership for membership in membership_dict.keys()}
        if slugs_of_found_memberships != slugs_of_given_memberships:
            return JsonResponse({
                'code': 'MR4',
                'reason': "Some of the provided slugs are improper. Provided {}, requested {}"
                          "".format(slugs_of_given_memberships, slugs_of_found_memberships)
            }, status=400)

        # Update the status of the memberships
        to_be_updated_memberships = []
        for membership in found_memberships:
            status = self.action_to_status_dict[membership_dict[str(membership.slug)]]
            membership.status = status
            to_be_updated_memberships.append(membership)
        AssociationMembership.objects.bulk_update(to_be_updated_memberships, ['status'])

        # Set all profiles to the preferred profile
        if user.preferred_profile is not None:
            redundant_profiles = {m.profile for m in found_memberships}
            redundant_profiles -= {user.preferred_profile}
            if redundant_profiles:
                to_be_updated_memberships = []
                for membership in found_memberships:
                    membership.profile = user.preferred_profile
                    to_be_updated_memberships.append(membership)
                AssociationMembership.objects.bulk_update(to_be_updated_memberships, ['profile'])
                for profile in redundant_profiles:
                    profile.delete()

        return JsonResponse({
            'status': 'success'
        }, status=200)

    def has_proper_data_format(self, data):
        for slug, action in data.items():
            # check slug
            if not isinstance(slug, uuid.UUID):
                try:
                    uuid.UUID(slug)
                except (AttributeError, ValueError):
                    return False
            # check action
            if action not in self.action_to_status_dict.keys():
                return False
        return True
