import json
import uuid
from json.decoder import JSONDecodeError

from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction
from django.http import HttpResponse
from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View
from rest_framework.permissions import IsAuthenticated

from apps.MySU.models import AssociationMembership
from apps.MySU.models import GroupMembership
from apps.MySU.models import Profile


@method_decorator(csrf_exempt, name='dispatch')
class ProfileMergeView(View):
    http_method_names = ['post']
    permission_classes = [IsAuthenticated]

    def post(self, request):
        user = request.user
        try:
            slug = json.loads(request.body.decode('utf-8'))["slug"]
        except JSONDecodeError:
            return JsonResponse(status=400, data={"error": "E1: Slug field needed, not supplied."})

        try:
            uuid.UUID(slug)
        except (AttributeError, ValueError):
            return JsonResponse(status=400, data={"error": "E2: Invalid slug given."})

        try:
            profile = Profile.objects.get(slug=slug, user=user)
        except ObjectDoesNotExist:
            return JsonResponse(status=400, data={"error": f"E3: No profile for this user with slug {slug}."})

        group_memberships = GroupMembership.objects.filter(user=user)
        association_memberships = AssociationMembership.objects.filter(user=user)

        to_update_group_memberships = []
        if group_memberships is not None:
            for membership in group_memberships:
                membership.profile = profile
                to_update_group_memberships.append(membership)

        to_update_association_memberships = []
        if association_memberships is not None:
            for membership in association_memberships:
                membership.profile = profile
                to_update_association_memberships.append(membership)
        try:
            with transaction.atomic():
                # update references to other profiles of the user to the selected profile
                if to_update_group_memberships:
                    GroupMembership.objects.bulk_update(to_update_group_memberships, ['profile'])
                if to_update_association_memberships:
                    AssociationMembership.objects.bulk_update(to_update_association_memberships, ['profile'])
                # set the selected profile as the preferred_profile
                user.preferred_profile = profile
                user.save()
                # Remove the now unused profiles
                Profile.objects.filter(user=user).exclude(slug=slug).delete()
        except Exception as e:
            return JsonResponse(status=400, data={"error": f"E4: {e}."})

        return HttpResponse(status=200)
