from django.db.models import Prefetch
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from url_filter import filtersets

from apps.MySU.models import Association
from apps.MySU.models import AssociationMembership
from apps.MySU.models import Group
from apps.MySU.models import MemberType
from apps.MySU.serializers.association import AssociationSerializer
from apps.MySU.serializers.association import AssociationSerializerList

"""
----------------------------------------------------------------------------------------------------------------------
 ASSOCIATION
----------------------------------------------------------------------------------------------------------------------

 GET:       Everyone
 PUT/PATCH: Board
 POST:      None
"""


class AssociationHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj)
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return False
        return True


class AssociationFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Association


class AssociationViewSet(viewsets.ModelViewSet):
    serializer_class = AssociationSerializerList
    queryset = Association.objects.all()
    lookup_field = 'slug'
    permission_classes = (IsAuthenticated, AssociationHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'head', 'options']
    ordering_fields = ['slug', 'name', 'short_name', 'founding_date', 'city', 'phone_number']
    ordering = ['slug']
    filter_class = AssociationFilterSet

    def get_serializer_class(self):
        if self.action == "list":
            return AssociationSerializerList
        return AssociationSerializer

    def get_queryset(self):
        queryset = Association.objects.all()
        if self.action == "list":
            subquery = MemberType.objects.filter(only_chooseable_by_board=False)
            queryset = queryset.prefetch_related(Prefetch('membertype_set', queryset=subquery, to_attr="public_membertypes"))
        elif self.action in ["retrieve", "update", 'partial_update']:
            subquery = AssociationMembership.objects.filter(user=self.request.user)
            queryset = queryset.prefetch_related(Prefetch('memberships', queryset=subquery, to_attr="my_membership"))

            subquery = Group.objects.filter(user=self.request.user, board_group=False)
            queryset = queryset.prefetch_related(Prefetch('groups', queryset=subquery, to_attr="my_committees"))

            subquery = Group.objects.filter(board_group=True, current_board=True)
            queryset = queryset.prefetch_related(Prefetch('groups', queryset=subquery, to_attr="current_board")) \
                .prefetch_related('current_board__group__groupmemberships__profile')
        return queryset
