import datetime

from django.core.exceptions import ValidationError
from django.db.models import Q
from guardian.shortcuts import get_objects_for_user
from rest_framework import status
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.request import Request
from rest_framework.response import Response
from url_filter import filtersets

from apps.MySU.mixins import GetSerializerClassMixin
from apps.MySU.models import Association
from apps.MySU.models import AssociationMembership
from apps.MySU.serializers.association_membership import AssociationMembershipSerializerBase
from apps.MySU.serializers.association_membership import AssociationMembershipSerializerCreate
from apps.MySU.serializers.association_membership import AssociationMembershipSerializerUpdate

"""
----------------------------------------------------------------------------------------------------------------------
 ASSOCIATIONMEMBERSHIP
----------------------------------------------------------------------------------------------------------------------

 GET:       Board or Self
 PUT/PATCH: Board or Self
 POST:      Self
"""


class AssociationMembershipHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association) or obj.user == request.user
        if request.method == "DELETE":                            # HANDLES THE DELETE
            return obj.user
        return True

    def has_permission(self, request, view):
        if request.method == "GET":                               # HANDLES THE LIST
            return True
        if request.method == "POST":                              # HANDLES THE CREATION/INSERTION
            return True
        return True


class AssociationMembershipFilterSet(filtersets.ModelFilterSet):

    class Meta:
        model = AssociationMembership


class AssociationMembershipViewSet(GetSerializerClassMixin, viewsets.ModelViewSet):
    queryset = AssociationMembership.objects.none()
    serializer_class = AssociationMembershipSerializerBase
    serializer_action_classes = {
        "update": AssociationMembershipSerializerUpdate,
        "partial_update": AssociationMembershipSerializerUpdate,
        "create": AssociationMembershipSerializerCreate
    }
    permission_classes = (IsAuthenticated, AssociationMembershipHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'delete', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug', 'status', 'date_joined', 'date_left', 'association']
    ordering = ['slug']
    filter_class = AssociationMembershipFilterSet

    def get_queryset(self):
        associations = get_objects_for_user(self.request.user, 'board', klass=Association)
        queryset = AssociationMembership.objects\
            .filter(Q(user=self.request.user) | Q(association__in=associations, financial_obligations_satisfied=False))\
            .select_related("type", "association", "profile")\
            .prefetch_related("specific_data")
        return queryset

    def destroy(self, request: 'Request', *args: 'Any', **kwargs: 'Any') -> 'Response':
        membership = self.get_object()
        # Only Pending and Rejected Memberships may be deleted
        if membership.status not in [AssociationMembership.Status.PENDING, AssociationMembership.Status.REJECTED]:
            return Response(
                status=status.HTTP_400_BAD_REQUEST,
                data={"status": "Only pending and rejected memberships can be deleted"}
            )
        self.perform_destroy(membership)
        return Response(status=status.HTTP_204_NO_CONTENT)
