from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_framework import status
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from url_filter import filtersets

from apps.MySU.models import Association
from apps.MySU.models import Group
from apps.MySU.models import GroupMembership
from apps.MySU.serializers.group import GroupSerializer


"""
----------------------------------------------------------------------------------------------------------------------
 GROUP
----------------------------------------------------------------------------------------------------------------------

 GET:       Board or Self
 PUT/PATCH: Board
 POST:      Board
"""


class GroupHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association)
        if request.method == "DELETE":
            return request.user.has_perm('board', obj.association)
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        return True


class GroupFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Group


class GroupViewSet(viewsets.ModelViewSet):
    queryset = Group.objects.none()
    lookup_field = 'slug'
    permission_classes = (IsAuthenticated, GroupHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'delete', 'head', 'options']
    ordering_fields = ['full_name', 'short_name', 'alternate_name', 'founding_date', 'dissolution_date']
    ordering = ['full_name']
    filter_class = GroupFilterSet

    def get_serializer_class(self):
        return GroupSerializer

    def get_queryset(self):
        associations = Association.objects.associations_where_user_is_member(self.request.user)
        queryset = Group.objects\
            .filter(Q(current_board=True) | Q(association__in=associations)) \
            .select_related("association", "parent_group")\
            .prefetch_related("groupobjectpermission_set__permission")\
            .prefetch_related("groupmemberships", "groupmemberships__profile")
        return queryset.distinct()

    def destroy(self, request: 'Request', *args: 'Any', **kwargs: 'Any') -> 'Response':
        group = self.get_object()
        django_group = group.django_group
        if Group.objects.filter(parent_group=group).exists():
            return Response(
                status=status.HTTP_400_BAD_REQUEST,
                data={"status": "This group is a parent group of another group, and thus cannot be deleted"}
            )
        if GroupMembership.objects.filter(django_group=django_group).exists():
            return Response(
                status=status.HTTP_400_BAD_REQUEST,
                data={"status": "Groups with members can not be deleted"}
            )
        self.perform_destroy(group)
        return Response(status=status.HTTP_204_NO_CONTENT)
