import uuid

from django.core.exceptions import ValidationError
from django.core.mail import send_mail
from django.db import models

from apps.MySU.models import AssociationMembership


class EnrollmentQuerySet(models.QuerySet):
    pass


class EnrollmentManager(models.Manager):
    def get_queryset(self):
        return EnrollmentQuerySet(self.model, using=self._db)


class Enrollment(models.Model):
    """
    In order to enroll for an event the member needs to enroll. This is done through this model.  It creates a link
    between the event, the enrollment option chosen and the member.
    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    email_send = models.BooleanField(default=False)

    association = models.ForeignKey("MySU.Association", on_delete=models.PROTECT)
    debt_entry = models.ForeignKey("financial.DebtEntry", related_name='enrollments', on_delete=models.SET_NULL, null=True, blank=True)
    event = models.ForeignKey("calendar.Event", related_name='enrollments', on_delete=models.PROTECT)
    enrollment_option = models.ForeignKey("calendar.EnrollmentOption", related_name='enrollments', on_delete=models.PROTECT)
    profile = models.ForeignKey("MySU.Profile", related_name='enrollments', on_delete=models.PROTECT)

    objects = EnrollmentManager()

    class Meta:
        verbose_name = "Enrollment"
        verbose_name_plural = "Enrollments"
        unique_together = ('profile', 'event')

    def __str__(self):
        return super().__str__()

    def clean(self):
        # if self.debt_collection.locked:
        #     raise ValidationError("Cannot change enrollment after debt collection is locked.")
        if self.event != self.enrollment_option.event:
            raise ValidationError("Enrollment option not linked to event")
        if self.association != self.event.association:
            raise ValidationError("Event not linked to association")
        if not self.event.public and not AssociationMembership.objects.is_current_member(association=self.association, profile=self.profile):
            raise ValidationError("Private events are for members only")

    def send_enroll_email(self, update_email_send_in_db=False):
        send_mail(
            f"Enrolled for {self.event.name}",
            f"Hi {self.profile.given_name}\n\n"
            f"You have succesfully enrolled for {self.event.name}.\n\n"
            f"Kind regards,\n\n"
            f"The MySU team",
            "no-reply@su.utwente.nl",
            [self.profile.email]
        )
        if update_email_send_in_db:
            self.email_send = True
            self.save()

    def send_unenroll_email(self):
        send_mail(
            f"Unenrolled for {self.event.name}",
            f"Hi {self.profile.given_name}\n\n"
            f"You have succesfully unenrolled for {self.event.name}.\n\n"
            f"Kind regards,\n\n"
            f"The MySU team",
            "no-reply@su.utwente.nl",
            [self.profile.email]
        )
