import uuid

from django.core.exceptions import ValidationError
from django.db import models

from apps.calendar.models import EventType
from apps.MySU.models import Association
from apps.MySU.models import Profile


class EventQuerySet(models.QuerySet):
    pass


class EventManager(models.Manager):
    def get_queryset(self):
        return EventQuerySet(self.model, using=self._db)


class Event(models.Model):
    """
    Event can be created through this model. An event can be either
     - Public       : public
     - Private      : private
     - Enrollable   : private and enrollable
     - Unenrollable : private, enrollable and unenrollable

    Public:
    When an event is public, then (un)enrollable should be false and enrollable_from, (un)enrollable_until None. Since
    public events can be seen by everyone, this ensures that no member outside of the association can enroll for it.

    Private:
    This is simply and event that is neither public nor enrollable.

    Enrollable:
    This is an event to which members of the association can enroll. An enrollable event must have dates set from when
    people can enroll and until when people can enroll. Logically it must hold that
     enrollable_from =< enrollable_until =< start_date

    Unenrollable:
    This is an enrollable event from which members of the association can also unenroll. This means there has to be a
    date from which people can no longer enroll. This can be the start date of the event, but also earlier when stuff
    needs to be ordered. Hence
     enrollable_from =< enrollable_until =< unenrollable_until =< start_date

    """
    slug = models.SlugField(default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=100, blank=False)
    description = models.TextField(blank=True, null=True)
    start_date = models.DateTimeField()
    end_date = models.DateTimeField()
    hidden_for_members = models.BooleanField(default=False)
    enrollable = models.BooleanField(default=False)
    enrollable_from = models.DateTimeField(null=True, blank=True)
    enrollable_until = models.DateTimeField(null=True, blank=True)
    unenrollable = models.BooleanField(default=False)
    unenrollable_until = models.DateTimeField(null=True, blank=True)
    max_number_of_enrollments = models.PositiveIntegerField(null=True, blank=True)
    public = models.BooleanField(default=False)
    location = models.CharField(max_length=100, blank=True)

    association = models.ForeignKey(Association, on_delete=models.PROTECT)
    type = models.ForeignKey(EventType, on_delete=models.PROTECT)
    debt_collection = models.ForeignKey("financial.DebtCollection", related_name='events', on_delete=models.SET_NULL, null=True, blank=True)
    attendants = models.ManyToManyField(Profile, through='Enrollment', related_name='enrolled_events')

    objects = EventManager()

    class Meta:
        verbose_name = "Event"
        verbose_name_plural = "Events"

    def __str__(self):
        return self.name

    def clean(self):
        if self.end_date < self.start_date:
            raise ValidationError("Event cannot end before it starts")
        if self.enrollable:
            if not self.enrollable_from:
                raise ValidationError("Enrollable events needs to have a enrollable_from date")
            if not self.enrollable_until:
                raise ValidationError("Enrollable events needs to have a enrollable_until date")
            if self.enrollable_until < self.enrollable_from:
                raise ValidationError("enrollable_from date needs to be after enrollable_until date")
            if self.enrollable_until > self.end_date:
                raise ValidationError("Event cannot allow enroll after it ends")
            if self.max_number_of_enrollments is None:
                raise ValidationError("Enrollable event needs to have a max number of enrollments specified")
        if self.unenrollable:
            if not self.enrollable:
                raise ValidationError("Only enrollable events can be unenrolled from")
            if not self.unenrollable_until:
                raise ValidationError("Unenrollable events needs to have a unenrollable_until date")
            if self.unenrollable_until < self.enrollable_from:
                raise ValidationError("Final unenroll date for enrollable event cannot be before enrollable_from date")
            if self.unenrollable_until > self.start_date:
                raise ValidationError("You cannot unenroll from an event after it starts")
        if self.association != self.type.association:
            raise ValidationError("EventType not linked to same association")
