from django.core.exceptions import ValidationError
from django.utils import timezone
from rest_flex_fields import FlexFieldsModelSerializer
from rest_framework import serializers

from apps.calendar.models import Enrollment
from apps.calendar.serializers.enrollment_option import EnrollmentOptionSerializer
from apps.calendar.serializers.event import EventSerializer
from apps.MySU.models import AssociationMembership
from apps.MySU.serializers.association import AssociationSerializer
from apps.MySU.serializers.base import BaseSerializer
from apps.MySU.serializers.profile import ProfileSerializer


class EnrollmentSerializer(FlexFieldsModelSerializer, BaseSerializer):
    send_email = serializers.BooleanField(default=False)
    given_name = serializers.ReadOnlyField(source='profile.given_name')
    surname = serializers.ReadOnlyField(source='profile.surname')

    class Meta:
        model = Enrollment
        exclude = ('debt_entry', )
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'event': {'lookup_field': 'slug'},
            'enrollment_option': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'profile': {'lookup_field': 'slug'},
        }
        create_only_fields = ['profile', 'association', 'event']
        nested_fields = ['association']
        expandable_fields = {
            "association": (AssociationSerializer, {"read_only": True}),
            'event': (EventSerializer, {"read_only": True}),
            'enrollment_option': (EnrollmentOptionSerializer, {"read_only": True}),
            'profile': (ProfileSerializer, {"read_only": True})
        }

    def validate(self, attrs):
        attrs = super(EnrollmentSerializer, self).validate(attrs)

        errors = {}
        if self.instance is None:
            if attrs['association'] != attrs['event'].association:
                errors["event"] = "Event doesn't belong to provided association"
            if attrs['event'] != attrs['enrollment_option'].event:
                errors["enrollment_option"] = "Enrollment Option doesn't belong to provided event"
            if attrs['event'].enrollable_until < timezone.now():
                errors["enrollable_until"] = "Cannot enroll after enrollable date"
            if attrs['event'].enrollable_from > timezone.now():
                errors["enrollable_from"] = "Cannot enroll before enrollable date"
            if not AssociationMembership.objects.is_current_member(association=attrs['association'], profile=attrs['profile']):
                errors["profile"] = "Profile is no member of association"
            if not (self.context['request'].user == attrs['profile'].user or self.context['request'].user.has_perm("board", attrs['association'])):
                errors["profile"] = "User is no board for association or does not own the provided profile"
        if errors:
            raise ValidationError(errors)
        return attrs

    def create(self, validated_data):
        send_email = validated_data.pop("send_email")
        enrollment = super().create(validated_data)
        if send_email:
            enrollment.send_enroll_email(update_email_send_in_db=True)
        return enrollment
