from typing import Any

from django.core.exceptions import ValidationError
from rest_flex_fields import FlexFieldsModelSerializer
from rest_framework import serializers

from apps.calendar.models import Event
from apps.MySU.serializers.base import BaseSerializer
from MySU.fields.association import AssociationField
from MySU.helper_functions import get_for_patch


class EventSerializer(FlexFieldsModelSerializer, BaseSerializer):
    association = AssociationField(view_name='association-detail', lookup_field='slug')
    type_name = serializers.ReadOnlyField(source='type.name')

    class Meta:
        model = Event
        fields = ('start_date', 'end_date', 'public', 'enrollable', 'enrollable_from', 'enrollable_until', 'unenrollable',
                  'unenrollable_until', 'max_number_of_enrollments', 'association', 'type', 'type_name', 'hidden_for_members',
                  'slug', 'url', 'name', 'description', 'location', 'debt_collection')
        lookup_field = 'slug'
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'enrollments': {'lookup_field': 'slug'},
            'enrollment_options': {'lookup_field': 'slug'},
            'type': {'lookup_field': 'slug'},
            'debt_collection': {'lookup_field': 'slug'},
        }
        expandable_fields = {
            'enrollments': ('apps.calendar.EnrollmentSerializer', {"many": True, "read_only": True, "nested": True}),
            'enrollment_options': ("apps.calendar.EnrollmentOptionSerializer", {"many": True, "read_only": True, "nested": True}),
            'type': ("apps.calendar.EventTypeSerializer", {"read_only": True, "nested": True})
        }
        nested_fields = ['association']

    def validate(self, attrs: Any, *args, **kwargs):
        attrs = super().validate(attrs)

        start_date = get_for_patch(self.instance, attrs, 'start_date')
        end_date = get_for_patch(self.instance, attrs, 'end_date')
        public = get_for_patch(self.instance, attrs, 'public')
        enrollable_old = getattr(self.instance, 'enrollable', None)
        enrollable = get_for_patch(self.instance, attrs, 'enrollable')
        enrollable_from = get_for_patch(self.instance, attrs, 'enrollable_from')
        enrollable_until = get_for_patch(self.instance, attrs, 'enrollable_until')
        unenrollable = get_for_patch(self.instance, attrs, 'unenrollable')
        unenrollable_until = get_for_patch(self.instance, attrs, 'unenrollable_until')
        max_number_of_enrollments = get_for_patch(self.instance, attrs, 'max_number_of_enrollments')
        association = get_for_patch(self.instance, attrs, 'association')
        type_ = get_for_patch(self.instance, attrs, 'type')

        if end_date < start_date:
            raise ValidationError("Event cannot end before it starts")
        if enrollable:
            if public:
                raise ValidationError("Public events cannot be enrollable")
            if not enrollable_from:
                raise ValidationError("Enrollable events need to have a enrollable_from date")
            if not enrollable_until:
                raise ValidationError("Enrollable events need to have a enrollable_until date")
            if enrollable_until < enrollable_from:
                raise ValidationError("enrollable_from date needs to be after enrollable_until date")
            if enrollable_until > end_date:
                raise ValidationError("Event cannot allow enroll after it ends")
            if max_number_of_enrollments is None:
                raise ValidationError("Enrollable events need to have a max number of enrollments specified")
        if enrollable_old and not enrollable:
            if self.instance.enrollment_options.exists():
                raise ValidationError({"enrollment options": "Before making the event unenrollable, remove the enrollment options"})
        if unenrollable:
            if not enrollable:
                raise ValidationError("Only enrollable events can be unenrolled from")
            if not unenrollable_until:
                raise ValidationError("Unenrollable events needs to have a unenrollable_until date")
            if unenrollable_until < enrollable_from:
                raise ValidationError("Final unenroll date for enrollable event cannot be before enrollable_from date")
            if unenrollable_until > start_date:
                raise ValidationError("You cannot unenroll from an event after it starts")
        if association != type_.association:
            raise ValidationError("EventType not linked to same association")
        return attrs
