from django.utils import timezone
from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_flex_fields import is_expanded
from rest_framework import status
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from url_filter import filtersets

from apps.calendar.models import Enrollment
from apps.calendar.serializers.enrollment import EnrollmentSerializer
from apps.MySU.models import Association
from MySU.mixins import BulkCreateModelMixin
from MySU.mixins import BulkDestroyModelMixin


"""
----------------------------------------------------------------------------------------------------------------------
 Enrollment
----------------------------------------------------------------------------------------------------------------------

 GET:       Board and self
 PUT/PATCH: Self
 POST:      Everyone
"""


class EnrollmentHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return obj.user == request.user
        if request.method == "DELETE":
            return True
        return True

    def has_permission(self, request, view):
        if request.method == "GET":   # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return True
        return True


class EnrollmentFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Enrollment


class EnrollmentViewSet(BulkDestroyModelMixin, BulkCreateModelMixin, viewsets.ModelViewSet):
    queryset = Enrollment.objects.none()
    serializer_class = EnrollmentSerializer
    permission_classes = (IsAuthenticated, EnrollmentHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'delete', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug']
    ordering = ['slug']
    filter_class = EnrollmentFilterSet

    def get_queryset(self):
        associations = get_objects_for_user(self.request.user, 'board', klass=Association)
        queryset = Enrollment.objects.filter(
            Q(profile=self.request.user.preferred_profile) | Q(association__in=associations)
        )
        for field in ['event', 'association', 'enrollment_option']:
            if is_expanded(self.request, field):
                queryset = queryset.select_related(field)
        return queryset

    def validate_delete_bulk(self, queryset):
        if queryset.values('association').distinct().count() > 1:
            return False, {"association": "Cannot delete enrollments for multiple association as the same time."}
        if not self.request.user.has_perm("board", Association.objects.get(pk=queryset[0].association_id)):
            return False, {"association": "Only a board member can do a bulk delete."}
        if queryset.filter(debt_entry__isnull=False).exists():
            return False, {"locked": "Query contains locked enrollements; these cannot be removed."}
        return True, {}

    def destroy(self, request: 'Request', *args: 'Any', **kwargs: 'Any') -> 'Response':
        enrollment = self.get_object()
        if not enrollment.event.unenrollable:
            return Response(
                status=status.HTTP_400_BAD_REQUEST,
                data={"status": "Cannot unenroll from event that doesn't allow any more for unenrollment"}
            )

        if enrollment.event.unenrollable_until <= timezone.now():
            return Response(
                status=status.HTTP_400_BAD_REQUEST,
                data={"status": "Only enrollments can be deleted before unenrollable until"}
            )
        enrollment.send_unenroll_email()
        self.perform_destroy(enrollment)
        return Response(status=status.HTTP_204_NO_CONTENT)
