from django.contrib.auth.models import Permission
from django.shortcuts import get_object_or_404
from django.utils import timezone
from guardian.shortcuts import get_objects_for_user
from guardian.shortcuts import Q
from rest_flex_fields import is_expanded
from rest_framework import status
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from url_filter import filtersets

from apps.calendar.models import Event
from apps.calendar.serializers.event import EventSerializer
from apps.MySU.models import Association

"""
----------------------------------------------------------------------------------------------------------------------
 Event
----------------------------------------------------------------------------------------------------------------------

 GET:       Everyone
 PUT/PATCH: Board
 POST:      Board
"""


class EventHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "GET":
            return True
        if request.method == "PATCH" or request.method == "PUT":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association)
        if request.method == "DELETE":
            return request.user.has_perm('board', obj.association)
        return False

    def has_permission(self, request, view):    # this is subject to change !
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        if request.method == "PUT" or request.method == "PATCH":
            return True
        return True

# and self.user.has_perm('board')


class EventFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Event


class EventViewSet(viewsets.ModelViewSet):
    queryset = Event.objects.none()
    permission_classes = (IsAuthenticated, EventHttpPermissions,)
    http_method_names = ['get', 'post', 'put', 'patch', 'delete', 'head', 'options']
    lookup_field = 'slug'
    ordering_fields = ['slug']
    ordering = ['slug']
    filter_class = EventFilterSet

    def get_serializer_class(self):
        return EventSerializer

    def get_queryset(self):
        associations = get_objects_for_user(self.request.user, 'board', klass=Association)
        queryset = Event.objects.filter(
            Q(public=True) | Q(association__users=self.request.user)
        ).filter(
            Q(hidden_for_members=False) | Q(association__in=associations)
        )
        for field in ['event', 'association', 'enrollment_option']:
            if is_expanded(self.request, field):
                queryset = queryset.select_related(field)
        return queryset.distinct()

    def patch(self, request, *args, **kwargs):
        instance = self.get_object()
        if not instance:
            return Response(status=status.HTTP_404_NOT_FOUND)
        serializer = self.get_serializer(instance, data=request.data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data, status=status.HTTP_200_OK)
        else:
            return Response(status=status.HTTP_400_BAD_REQUEST)

    def destroy(self, request: 'Request', *args: 'Any', **kwargs: 'Any') -> 'Response':
        event = self.get_object()
        if event.attendants.exists():
            return Response(
                status=status.HTTP_400_BAD_REQUEST,
                data={"status": "Only events without enrollments can be deleted"}
            )
        if event.start_date <= timezone.now():
            return Response(
                status=status.HTTP_400_BAD_REQUEST,
                data={"status": "Only events that have not started can be deleted"}
            )
        self.perform_destroy(event)
        return Response(status=status.HTTP_204_NO_CONTENT)
