import datetime
from decimal import Decimal

from dateutil.relativedelta import relativedelta
from dateutil.tz import UTC
from django.core.exceptions import ValidationError
from django.db.models import Exists
from django.db.models import OuterRef
from rest_flex_fields import FlexFieldsModelSerializer
from rest_framework import serializers

from apps.calendar.models import Enrollment
from apps.calendar.models import Event
from apps.calendar.serializers import EventSerializer
from apps.financial.models import DebtCollection
from apps.financial.models import DebtEntry
from apps.financial.models import Invoice
from apps.financial.models import MembershipFee
from apps.financial.serializers.debt_entry import DebtEntrySerializer
from apps.MySU.models import AssociationMembership
from apps.MySU.serializers import AssociationSerializer
from apps.MySU.serializers.base import BaseSerializer
from MySU.helper_functions import get_for_patch


class DebtCollectionSerializerList(BaseSerializer):
    class Meta:
        model = DebtCollection
        fields = ['url', 'slug', 'debt_collection_date', 'association', 'amount', 'locked']
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
            'debt_entries': {'lookup_field': 'slug'},
        }


class DebtCollectionSerializer(FlexFieldsModelSerializer, serializers.HyperlinkedModelSerializer):
    # field determining what serializer should do
    include_events = serializers.HyperlinkedRelatedField(many=True, write_only=True, view_name='event-detail',
                                                         queryset=Event.objects.filter(end_date__lt=datetime.date.today()),
                                                         lookup_field="slug", default=[])
    # include_events = EventSerializer(many=True, write_only=True, required=False)
    include_membership_fees = serializers.BooleanField(write_only=True, default=False)
    include_invoices = serializers.BooleanField(write_only=True, default=False)
    ignore_threshold = serializers.BooleanField(write_only=True, default=False)

    class Meta:
        model = DebtCollection
        fields = '__all__'
        extra_kwargs = {
            'url': {'lookup_field': 'slug'},
            'debt_entries': {'lookup_field': 'slug'},
            'association': {'lookup_field': 'slug'},
        }
        read_only_fields = ['amount', 'creation_date']
        expandable_fields = {
            "debt_entries": (DebtEntrySerializer, {"many": True, "read_only": True, "nested": True}),
            'association': (AssociationSerializer, {"read_only": True, "nested": True}),
            'events': (EventSerializer, {"many": True, "nested": True}),
        }

    def validate(self, attrs):
        one_second_from_now = datetime.datetime.now(tz=UTC) + relativedelta(seconds=1)
        creation_date = get_for_patch(self.instance, {'creation_date': one_second_from_now}, 'creation_date')
        debt_collection_date = get_for_patch(self.instance, attrs, 'debt_collection_date')
        association = get_for_patch(self.instance, attrs, 'association')

        errors = {}
        if not self.context['request'].user.has_perm("board", association):
            errors["association"] = "User is no board for association"
        if attrs.get('include_events', None):
            mismatched_events = []
            for event in attrs['include_events']:
                if event.association != association:
                    mismatched_events.append(event)
            if mismatched_events:
                errors["include_events"] = f"Not all supplied events are associated with the supplied association."
        if creation_date >= debt_collection_date:
            errors["debt_collection_date"] = f"Debt collection date, {creation_date}, should be after the creation date, {debt_collection_date}."
        if errors:
            raise ValidationError(errors)

        return attrs

    def update(self, instance, validated_data):
        """ Update DebtCollection

        Most fields of debt collection are not to be changed after creation .
        """
        # Update should not use these
        validated_data.pop("include_invoices", None)
        validated_data.pop("include_events", None)
        validated_data.pop("include_membership_fees", None)
        validated_data.pop("ignore_threshold", None)
        # Do the default update
        return super(DebtCollectionSerializer, self).update(instance, validated_data)

    def create(self, validated_data):
        """ Create DebtCollection and related debt entries

        DebtCollection is a collection of debt entries.
        Debt entries currently consist of enrollments and membershipfees.
        The debt entries in the debt collection will be created here too.

        Deleting a debt collection will also create debt_entries without debt_collection.
        These must be linked to the new debt_collection.
        """
        include_invoices = validated_data.pop("include_invoices", None)
        events = validated_data.pop("include_events", None)
        include_membership_fees = validated_data.pop("include_membership_fees", None)
        ignore_threshold = validated_data.pop("ignore_threshold", None)

        debt_collection = super(DebtCollectionSerializer, self).create(validated_data)
        association = validated_data["association"]

        to_be_created_or_updated_debt_entries = list(DebtEntry.objects.filter(debt_collection__isnull=True))

        # ---- Invoices ----
        if include_invoices:
            to_be_updated_invoices = Invoice.objects.filter(debt_entry__isnull=True)

            for invoice in to_be_updated_invoices:
                debt_entry = None
                # Check if there is already a debt_entry for the user associatied to the invoice
                # If found, select debt entry
                for entry in to_be_created_or_updated_debt_entries:
                    if entry.profile == invoice.profile:
                        debt_entry = entry
                        debt_entry.debt_collection = debt_collection
                        break
                # If not, create a debt entry
                if debt_entry is None:
                    debt_entry = DebtEntry(
                        association=association,
                        debt_collection=debt_collection,
                        profile=invoice.profile,
                        bank_account=invoice.profile.bank_account,
                        total_price=Decimal(0)
                    )
                    to_be_created_or_updated_debt_entries.append(debt_entry)
                # Set the debt entry on the invoice and update the total for the entry
                invoice.debt_entry = debt_entry
                debt_entry.total_price = debt_entry.total_price + invoice.price

        # ---- Enrollments ----
        if events:
            to_be_updated_events = events
            to_be_updated_enrollments = Enrollment.objects.filter(association=association, event__in=to_be_updated_events)

            for enrollment in to_be_updated_enrollments:
                debt_entry = None
                # Check if there is already a debt_entry for the user associatied to the enrollment
                # If found, select debt entry
                for entry in to_be_created_or_updated_debt_entries:
                    if entry.profile == enrollment.profile:
                        debt_entry = entry
                        debt_entry.debt_collection = debt_collection
                        break
                # If not, create a debt entry
                if debt_entry is None:
                    debt_entry = DebtEntry(
                        association=association,
                        debt_collection=debt_collection,
                        profile=enrollment.profile,
                        bank_account=enrollment.profile.bank_account,
                        total_price=Decimal(0)
                    )
                    to_be_created_or_updated_debt_entries.append(debt_entry)
                # Set the debt entry on the enrollment and update the total for the entry
                enrollment.debt_entry = debt_entry
                debt_entry.total_price = debt_entry.total_price + enrollment.enrollment_option.participation_fee
            for event in to_be_updated_events:
                event.debt_collection = debt_collection

        # ---- Membership fees ----
        if include_membership_fees:
            to_be_created_membershipfees = []

            today = datetime.date.today()
            current_payment_year = today.year if today >= association.full_year_membership_start_date else today.year - 1

            membership_fee_query = MembershipFee.objects.filter(association=association, association_membership=OuterRef("pk"), payments=current_payment_year)
            memberships_without_membership_fee = AssociationMembership.objects.filter(association=association).current().filter(~Exists(membership_fee_query))

            for membership in memberships_without_membership_fee:
                membershipfee = MembershipFee(
                    association=association,
                    membertype=membership.type,
                    association_membership=membership,
                    fee=membership.type.membership_fee,
                    payments=current_payment_year
                )
                debt_entry = None
                # Check if there is already a debt_entry for the user
                # If found, select the debt entry
                for entry in to_be_created_or_updated_debt_entries:
                    if entry.profile == membership.profile:
                        debt_entry = entry
                        debt_entry.debt_collection = debt_collection
                        break
                # If not, create a debt entry
                if debt_entry is None:
                    debt_entry = DebtEntry(
                        association=association,
                        debt_collection=debt_collection,
                        profile=membership.profile,
                        bank_account=membership.profile.bank_account,
                        total_price=Decimal(0)
                    )
                    to_be_created_or_updated_debt_entries.append(debt_entry)
                # Set the debt entry on the fee and update the total for the entry
                membershipfee.debt_entry = debt_entry
                debt_entry.total_price = debt_entry.total_price + membership.type.membership_fee

        # ---- Debt Collection Threshold ----
        # Before creating check whether something has to be done with the debt collection threshold.
        # If so, remove the entries from the debt collection.
        if not ignore_threshold:
            for debt_entry in to_be_created_or_updated_debt_entries:
                if debt_entry.total_price < association.debt_collection_threshold:
                    debt_entry.debt_collection = None

        # Split entries into those that need updating and those that need to be created
        to_be_created_debt_entries = []
        to_be_updated_debt_entries = []
        for debt_entry in to_be_created_or_updated_debt_entries:
            if debt_entry.id:
                to_be_updated_debt_entries.append(debt_entry)
            else:
                to_be_created_debt_entries.append(debt_entry)

        # Do bulk updates/creates. NB: bulk updates/creates bypass the clean/validate so make sure all the input is correct
        created_debt_entries = DebtEntry.objects.bulk_create(to_be_created_debt_entries)
        DebtEntry.objects.bulk_update(to_be_updated_debt_entries, ["debt_collection", "total_price"])

        # Updated the invoice, events, enrollments and membership fees with the link to their debt collection
        # When establishing the link to debt collection/entry earlier, some of the debt entries did not have a pk.
        # We need to fix that to make sure that the foreignkey relations are set properly.
        created_or_updated_debt_entries = created_debt_entries + to_be_created_debt_entries
        if include_invoices:
            # fix pk of entries
            for invoice in to_be_updated_invoices:
                for entry in created_or_updated_debt_entries:
                    if invoice.debt_entry.slug == entry.slug:
                        invoice.debt_entry = entry
                        break
            # save
            Invoice.objects.bulk_update(to_be_updated_invoices, ["debt_entry"])
        if events:
            # fix pk of entries
            for enrollment in to_be_updated_enrollments:
                for entry in created_or_updated_debt_entries:
                    if enrollment.debt_entry.slug == entry.slug:
                        enrollment.debt_entry = entry
                        break
            # save
            Event.objects.bulk_update(to_be_updated_events, ["debt_collection"])
            Enrollment.objects.bulk_update(to_be_updated_enrollments, ["debt_entry"])
        if include_membership_fees:
            # fix pk of entries
            for fee in to_be_created_membershipfees:
                for entry in created_or_updated_debt_entries:
                    if fee.debt_entry.slug == entry.slug:
                        fee.debt_entry = entry
                        break
            # save
            MembershipFee.objects.bulk_create(to_be_created_membershipfees)

        # Bulk creation/update doesn't seem to trigger the computed fields, so trigger manually.
        debt_collection.save()
        return debt_collection
