from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Prefetch
from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from rest_framework import permissions
from rest_framework.views import APIView

from apps.calendar.models import Enrollment
from apps.financial.models import DebtCollection
from apps.financial.models import DebtEntry
from apps.financial.models import Invoice
from apps.financial.models import MembershipFee


@method_decorator(csrf_exempt, name='dispatch')
class DebtCollectionDetail(APIView):
    http_method_names = ['get']
    permission_classes = (permissions.IsAuthenticated,)

    def get(self, request, **kwargs):
        debt_collection_slug = kwargs["slug"]

        try:
            debt_collection = DebtCollection.objects.get(slug=debt_collection_slug)
        except ObjectDoesNotExist:
            return JsonResponse(status=404, data="E1: No debt collection with slug {}.".format(debt_collection_slug))

        if not request.user.has_perm('board', debt_collection.association):
            return JsonResponse(status=403, data="E2: No permission to access debt collection with slug {}.".format(debt_collection_slug))

        invoices_queryset = Invoice.objects\
            .filter(association=debt_collection.association)
        enrollments_queryset = Enrollment.objects\
            .filter(association=debt_collection.association)\
            .select_related("enrollment_option", "event")
        membership_fees_queryset = MembershipFee.objects\
            .filter(association=debt_collection.association)\
            .select_related("membertype")
        debt_entries = DebtEntry.objects\
            .filter(debt_collection=debt_collection)\
            .prefetch_related(
                Prefetch("invoices", queryset=invoices_queryset),
                Prefetch("enrollments", queryset=enrollments_queryset),
                Prefetch("membership_fees", queryset=membership_fees_queryset),
            )\
            .all()
            # .select_related("invoices", "enrollments__event", "enrollments__enrollment_option", "membership_fees__membertype")

        events = {}
        invoices = 0
        membership_fees = {}
        for debt_entry in debt_entries:
            for enrollment in debt_entry.enrollments.all():
                id_ = enrollment.event.slug + enrollment.enrollment_option.slug
                try:
                    events[id_]["number"] += 1
                except KeyError:
                    events[id_] = {
                        "number": 1,
                        "enrollment_option": enrollment.enrollment_option.name,
                        "fee": enrollment.enrollment_option.participation_fee,
                        "event": enrollment.event.name,
                        "slug": enrollment.event.slug
                    }
            for invoice in debt_entry.invoices.all():
                invoices += invoice.price
            for membership_fee in debt_entry.membership_fees.all():
                member_type = membership_fee.type
                try:
                    membership_fees[member_type]["number"] += 1
                except KeyError:
                    membership_fees[member_type] = {
                        "number": 1,
                        "fee": member_type.membership_fee,
                        "type": member_type.type,
                        "slug": member_type.slug
                    }

        return JsonResponse(status=200, data={
            "events": list(events.values()),
            "products": [],
            "invoices": float(invoices),
            "membership_fees": list(membership_fees.values())
        })
