from django.core import serializers
from django.core.exceptions import ObjectDoesNotExist
from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from rest_framework import permissions
from rest_framework.views import APIView

from apps.calendar.models import Enrollment
from apps.financial.models import DebtEntry
from apps.financial.models import Invoice
from apps.financial.models import MembershipFee


@method_decorator(csrf_exempt, name='dispatch')
class DebtEntryDetail(APIView):
    http_method_names = ['get']
    permission_classes = (permissions.IsAuthenticated,)

    def get(self, request, **kwargs):
        debt_entry_slug = kwargs["slug"]

        try:
            debt_entry = DebtEntry.objects.get(slug=debt_entry_slug)
        except ObjectDoesNotExist:
            return JsonResponse(status=404, data="E1: No debt entry with slug {}.".format(debt_entry_slug), safe=False)

        if not ( request.user.has_perm('board', debt_entry.association) or debt_entry.profile.user == request.user):
            return JsonResponse(status=403, data="E2: No permission to access debt entry with slug {}.".format(debt_entry_slug), safe=False)

        invoices = Invoice.objects\
            .filter(debt_entry=debt_entry)\
            .all()
        membership_fees_queryset = MembershipFee.objects\
            .filter(debt_entry=debt_entry)\
            .select_related("membertype")
        enrollments_queryset = Enrollment.objects\
            .filter(debt_entry=debt_entry)\
            .select_related("enrollment_option", "event")

        membership_fees = []
        for membership_fee in membership_fees_queryset.all():
            membership_fees.append({
                "slug": membership_fee.slug,
                "type": membership_fee.membertype.type,
                "fee": membership_fee.membertype.membership_fee
            })

        enrollments = []
        for enrollment in enrollments_queryset.all():
            enrollments.append({
                "slug": enrollment.slug,
                "event": enrollment.event.name,
                "enrollment_option": enrollment.enrollment_option.name,
                "fee": enrollment.enrollment_option.participation_fee
            })

        return JsonResponse(status=200, data={
            "events": enrollments,
            "products": [],
            "invoices": list(invoices.values()),
            "membership_fees": membership_fees
        })
