from django.db.models import Q
from guardian.shortcuts import get_objects_for_user
from rest_framework import status
from rest_framework import viewsets
from rest_framework.permissions import BasePermission
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from url_filter import filtersets

from apps.financial.models import Invoice
from apps.financial.serializers.invoice import InvoiceSerializer
from apps.MySU.models import Association
from MySU.mixins import BulkCreateModelMixin
from MySU.mixins import BulkDestroyModelMixin

"""
----------------------------------------------------------------------------------------------------------------------
 Invoice
----------------------------------------------------------------------------------------------------------------------

 GET:       Board or self
 PUT/PATCH: Board
 POST:      Board
"""


class InvoiceHttpPermissions(BasePermission):
    def has_object_permission(self, request, view, obj):
        if request.method == "PATCH" or request.method == "PUT":
            return request.user.has_perm('board', obj.association)
        # This will not be called for the bulk delete operation.
        if request.method == "DELETE":  # HANDLES THE UPDATE
            return request.user.has_perm('board', obj.association)
        return True

    def has_permission(self, request, view):
        if request.method == "GET":  # HANDLES THE LIST
            return True
        if request.method == "POST":  # HANDLES THE CREATION/INSERTION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        if request.method == "DELETE":  # HANDLES THE DELETION
            return get_objects_for_user(request.user, 'MySU.board').count() > 0
        return True


class InvoiceFilterSet(filtersets.ModelFilterSet):
    class Meta:
        model = Invoice


class InvoiceViewSet(BulkCreateModelMixin, BulkDestroyModelMixin, viewsets.ModelViewSet):
    queryset = Invoice.objects.none()
    serializer_class = InvoiceSerializer
    permission_classes = (IsAuthenticated, InvoiceHttpPermissions,)
    http_method_names = ['get', 'put', 'patch', 'post', 'head', 'options', 'delete']
    lookup_field = 'slug'
    ordering_fields = ['slug', 'association']
    ordering = ['slug']
    filter_class = InvoiceFilterSet

    def get_queryset(self):
        queryset = Invoice.objects.filter(
            Q(association__in=get_objects_for_user(self.request.user, 'board', Association)) | Q(profile__user=self.request.user)
        ).select_related("debt_entry__debt_collection")
        return queryset

    def validate_delete_bulk(self, queryset):
        if queryset.filter(debt_entry__isnull=False).exists():
            return False, {"locked": "Query contains locked invoices; these cannot be removed."}
        return True, {}

    def destroy(self, request: 'Request', *args: 'Any', **kwargs: 'Any') -> 'Response':
        invoice = self.get_object()
        if invoice.locked:
            return Response(
                status=status.HTTP_400_BAD_REQUEST,
                data={"status": "Locked invoice cannot be removed."}
            )
        self.perform_destroy(invoice)
        return Response(status=status.HTTP_204_NO_CONTENT)
