#!/bin/bash

# ------------------------------------------
# Handling parameters
# ------------------------------------------
ARGS=$(getopt -o e:chsn --long env:,conda,help,skip,no-fixture -- "$@")
eval set -- "$ARGS"
while true; do
    case "$1" in
        -e | --env ) ENV=$2; shift 2 ;;
        -c | --conda ) CONDA=true; shift ;;
        -h | --help ) HELP=true; shift ;;
        -n | --no-fixture ) FIXTURE=false; shift ;;
        -s | --skip ) SKIP=true; shift ;;
        -- ) shift; break ;;
        * ) break ;;
    esac
done
ENV=${ENV:-local}
CONDA=${CONDA:-false}
HELP=${HELP:-false}
SKIP=${SKIP:-false}
FIXTURE=${FIXTURE:-true}

declare -A ENVS
ENVS=(['test']=1 ['prod']=1 ['dev']=1 ['pilot']=1 ['local']=1)
if [[ ! ${ENVS[$ENV]} ]]; then
    echo
    echo -e "\e[1;31mSupplied env is not supported\e[0m"
    echo "Valid options are (test, prod, dev, pilot, local)"
    echo
    exit 1
fi

if $HELP; then
    cat << "EOF"
 This script will take the necessary steps to set up an proper virtualenv and working database.

 The following is done in order:
  - Check if config file needs to be filled, and if so, fill it will user input.
  - Create env
  - Activate env
  - Git pull --rebase the latest changes
  - Install requirements for specified env
  - Execute migrations
  - Collect static, if needed

 The following parameters are able to be used:
  -c | --conda          Make an anaconda env instead of an virtualenv
  -e | --env            Specify which environment should be installed, default local
  -h | --help           Raise this help
  -n | --no-fixture     When set, do not load the fixtures
  -s | --skip           Skip the git pull --rebase
EOF
    exit 0
else
# ------------------------------------------
# Welcome greeting
# ------------------------------------------
    cat << "EOF"
                                                     ___
                                                  ,o88888
                                               ,o8888888'
                         ,:o:o:oooo.        ,8O88Pd8888"
                     ,.::.::o:ooooOoOoO. ,oO8O8Pd888'"
                   ,.:.::o:ooOoOoOO8O8OOo.8OOPd8O8O"
                  , ..:.::o:ooOoOOOO8OOOOo.FdO8O8"
                 , ..:.::o:ooOoOO8O888O8O,COCOO"
                , . ..:.::o:ooOoOOOO8OOOOCOCO"
                 . ..:.::o:ooOoOoOO8O8OCCCC"o
                    . ..:.::o:ooooOoCoCCC"o:o
                    . ..:.::o:o:,cooooCo"oo:o:
                 `   . . ..:.:cocoooo"'o:o:::'
                 .`   . ..::ccccoc"'o:o:o:::'
                :.:.    ,c:cccc"':.:.:.:.:.'
              ..:.:"'`::::c:"'..:.:.:.:.:.'
            ...:.'.:.::::"'    . . . . .'
           .. . ....:."' `   .  . . ''
         . . . ...."'
         .. . ."'
        .
EOF
fi
echo
echo "Welcome to the MySU installer program!"

# ------------------------------------------
# Detection of platform
# ------------------------------------------
IS_LINUX=false      # This includes WSL
IS_OSX=false
IS_WINDOWS=false
PLATFORM="$(expr substr $(uname -s) 1 5)"
if [[ $PLATFORM == "Linux" ]]; then
    IS_LINUX=true
fi
if [[ $PLATFORM == "Darwi" ]]; then # OSX starts with Darwin
    IS_OSX=true
fi
if [[ $PLATFORM == "MINGW" ]]; then
    IS_WINDOWS=true
fi

if [[ ! ($IS_LINUX == true || $IS_OSX == true || $IS_WINDOWS == true) ]]; then
    echo
    echo -e "\e[1;31mPlatform detection went haywire.\e[0m"
    echo -e "\e[1;31mCode thinks its neither Linux, OS X nor Windows, but $PLATFORM.\e[0m"
    echo
    exit 1
fi

# ------------------------------------------
# Checking if in correct folder
# ------------------------------------------
if [[ ! -e $PWD/manage.py ]]; then
    echo
    echo -e "\e[1;31mMake sure to execute this in the top directory.\e[0m"
    echo
    exit 1
fi

# ------------------------------------------
# Checking if dependencies satisfied
# ------------------------------------------
# +++++++++++++++++++++
# CONDA
# +++++++++++++++++++++
WINDOWS_CONDA=false
LINUX_CONDA=false
if $CONDA; then
    if command -v conda.exe >& /dev/null; then
        WINDOWS_CONDA=true
    fi
    if command -v conda >& /dev/null; then
        LINUX_CONDA=true
    fi

    if [[ ! ( $WINDOWS_CONDA || $LINUX_CONDA ) ]]; then
      echo "You don't have a version of conda installed; neither a windows version nor a linux version"
      exit 1;
    fi

    if [[ $IS_OSX || $IS_LINUX ]]; then
        if $LINUX_CONDA; then
            WINDOWS_CONDA=false
        else
            echo "Your version of conda (WINDOWS), does not match the terminal(LINUX/OSX) you are using."
            exit 1
        fi
    fi
    if $IS_WINDOWS; then
        if $WINDOWS_CONDA; then
            LINUX_CONDA=false
        else
            echo "Your version of conda (LINUX), does not match the terminal(WINDOWS) you are using."
            exit 1
        fi
    fi
# +++++++++++++++++++++
# Virtualenv
# +++++++++++++++++++++
else
    USE_PYTHON3=true
    if ! command -v python3 >& /dev/null; then
        USE_PYTHON3=false
    fi
    if ! command -v python >& /dev/null && ! $USE_PYTHON3; then
        echo
        echo -e "\e[1;31mMake sure you have python3.6 or higher installed and appended to your PATH.\e[0m"
        echo
        exit 1
    fi
    if $USE_PYTHON3; then
        PYTHON_VERSION=$(python3 --version 2>&1) || exit 1
    else
        PYTHON_VERSION=$(python --version 2>&1) || exit 1
    fi
    if [[ "$PYTHON_VERSION" =~ ^3\.[6-9]\.[0-9]$ ]]; then
        echo -e "\e[1;31mYou have $PYTHON_VERSION installed, please install python3.6 or higher.\e[0m"
        echo
        exit 1
    fi

    USE_PIP3=true
    if ! command -v pip3 >& /dev/null; then
        USE_PIP3=false
    fi
    if ! command -v pip >& /dev/null && ! $USE_PIP3; then
        echo -e "\e[1;31mPlease make sure you have pip3 installed.\e[0m"
        echo
        exit 1
    fi
    if ! command -v virtualenv >& /dev/null; then
        echo -e "\e[1;31mPlease make sure you have virtualenv installed.\e[0m"
        echo
        exit 1
    fi
fi

# ------------------------------------------
# Configuring settings
# ------------------------------------------
echo
echo -e "\e[1;32mConfiguring settings\e[0m"
echo

if [[ $ENV == 'local' ]]; then
    DO_CONFIG=true
    if [[ -e MySU/settings/local.py ]]; then
        DO_CONFIG=false
        echo -e -n "Config already done, reconfigure? [\e[33mYes\e[0m] "
        read -r REDO_CONFIG
        if [[ -z "$REDO_CONFIG" ]] || [[ "$REDO_CONFIG" = "Yes" ]]; then
            DO_CONFIG=true
        fi
    fi
else
    DO_CONFIG=false
fi

if $DO_CONFIG; then
    echo
    echo -e "\e[1;32mPlease fill in the questions to configure your local settings properly\e[0m"
    echo

    echo -e "Database settings:"
    echo -e -n "MySU database: "
    read -r DATABASE
    echo -e -n "MySU database user: "
    read -r USER
    echo -e -n "MySU database password: "
    read -r -s PASSWORD
    echo -e -n "MySU database schema: (django)"
    read -r SCHEMA
    if [[ -z "$SCHEMA" ]]; then
        SCHEMA=django
    fi
    echo -e -n "MySU database host: (localhost)"
    read -r HOST
    if [[ -z "$HOST" ]]; then
        HOST=localhost
    fi
    echo -e -n "MySU database port: (5432)"
    read -r PORT
    if [[ -z "$PORT" ]]; then
        PORT=5432
    fi

    {
      echo "from MySU.settings.develop import *"
      echo ""
      echo "DATABASES = {"
      echo "    'default': {"
      echo "        'ENGINE': 'django.db.backends.postgresql_psycopg2',"
      echo "        'OPTIONS': {"
      echo "            'options': '-c search_path=$SCHEMA,public'"
      echo "        },"
      echo "        'NAME': '$DATABASE',"
      echo "        'USER': '$USER',"
      echo "        'PASSWORD': '$PASSWORD',"
      echo "        'HOST': '$HOST',"
      echo "        'PORT': '$PORT',"
      echo "    }"
      echo "}"
      echo ""
      echo "CSRF_TRUSTED_ORIGINS = ['localhost, 127.0.0.1']"
      echo "SESSION_COOKIE_DOMAIN = '127.0.0.1'"
      echo ""
    } > MySU/settings/local.py
fi

# ------------------------------------------
# Setting up environment
# ------------------------------------------
echo
echo -e "\e[1;32mSetting up virtual environment\e[0m"
echo
# +++++++++++++++++++++
# CONDA
# +++++++++++++++++++++
if $CONDA; then
    if $WINDOWS_CONDA; then
        conda.exe create --prefix ./venv python=3.8 --yes || exit 1
    fi
    if $LINUX_CONDA; then
        conda create --prefix ./venv python=3.8 --yes || exit 1
    fi
# +++++++++++++++++++++
# Virtualenv
# +++++++++++++++++++++
else
    if $IS_OSX || $IS_LINUX; then
        if $USE_PYTHON3; then
            python3 -m virtualenv venv --python="$(command -v python3)" || exit 1
        else
            python -m virtualenv venv --python="$(command -v python)" || exit 1
        fi
    fi
    if $IS_WINDOWS; then
        if $USE_PYTHON3; then
            python3 -m virtualenv venv --python="$(command -v python3).exe" || exit 1
        else
            python -m virtualenv venv --python="$(command -v python).exe" || exit 1
        fi
    fi
fi

# ------------------------------------------
# Activating enviroment
# ------------------------------------------
echo
echo -e "\e[1;32mActivating virtual environment\e[0m"
echo
# +++++++++++++++++++++
# CONDA
# +++++++++++++++++++++
if $CONDA; then
    if $WINDOWS_CONDA; then
        activate venv || exit 1
    fi
    if $LINUX_CONDA; then
        source activate venv || exit 1
    fi
# +++++++++++++++++++++
# Virtualenv
# +++++++++++++++++++++
else
    if [[ $IS_OSX || $IS_LINUX ]]; then
        source venv/bin/activate || exit 1
    fi
    if $IS_WINDOWS; then
        source venv/Scripts/activate || exit 1
    fi
fi

# ------------------------------------------
# Updating env
# ------------------------------------------
if $SKIP; then
    bin/update_env.sh --skip --env=$ENV || exit 1
else
    bin/update_env.sh --env=$ENV || exit 1
fi
# ------------------------------------------
# Loading fixtures
# ------------------------------------------

if $FIXTURE; then
  if [[ $ENV == 'test' ]]; then
    bin/load_fixtures.sh -t || exit 1
  else
    bin/generate_fixtures.sh || exit 1
    bin/load_fixtures.sh || exit 1
  fi
fi
# ------------------------------------------
# Finishing
# ------------------------------------------
echo
echo -e "\e[1;32mDONE!\e[0m"
echo
# +++++++++++++++++++++
# CONDA
# +++++++++++++++++++++
if $CONDA; then
    if $WINDOWS_CONDA; then
        echo -e "To activate your conda env from bash use '\e[33msource activate venv\e[0m'"
    fi
    if $LINUX_CONDA; then
        echo -e "To activate your conda env use '\e[33msource activate venv\e[0m'"
    fi
# +++++++++++++++++++++
# Virtualenv
# +++++++++++++++++++++
else
    if [[ $IS_OSX || $IS_LINUX ]]; then
        echo -e "To activate your virtualenv use '\e[33msource venv/bin/activate\e[0m'"
    fi
    if $IS_WINDOWS; then
        echo -e "To activate your virtualenv use '\e[33msource venv/Scripts/activate\e[0m'"
    fi
fi
# +++++++++++++++++++++
# Virtualenv and CONDA
# +++++++++++++++++++++
echo -e "Once your virtualenv is activated use '\e[33mpython manage.py runserver\e[0m' to start a local server."
if $FIXTURE; then
    echo -e "You can sign in using '\e[33ms1234567\e[0m' with '\e[33mSUNetwork\e[0m'"
else
    echo -e "You can create a superuser using '\e[33mpython manage.py createsuperuser\e[0m'"
fi

cat << "EOF"

             . .                     -:-             .  .  .
            .'.:,'.        .  .  .     ' .           . \ | / .
            .'.;.`.       ._. ! ._.       \          .__\:/__.
             `,:.'         ._\!/_.                     .';`.      . ' .
             ,'             . ! .        ,.,      ..======..       .:.
            ,                 .         ._!_.     ||::: : | .        ',
     .====.,                  .           ;  .~.===: : : :|   ..===.
     |.::'||      .=====.,    ..=======.~,   |"|: :|::::::|   ||:::|=====|
  ___| :::|!__.,  |:::::|!_,   |: :: ::|"|l_l|"|:: |:;;:::|___!| ::|: : :|
 |: :|::: |:: |!__|; :: |: |===::: :: :|"||_||"| : |: :: :|: : |:: |:::::|
 |:::| _::|: :|:::|:===:|::|:::|:===F=:|"!/|\!"|::F|:====:|::_:|: :|::__:|
 !_[]![_]_!_[]![]_!_[__]![]![_]![_][I_]!//_:_\\![]I![_][_]!_[_]![]_!_[__]!
-----------------------------------"---''''```---"------------------------
 _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ |= _ _:_ _ =| _ _ _ _ _ _ _ _ _ _ _ _
           Congratz with             |=    :    =| your MySU environment!!
_____________________________________L___________J________________________
--------------------------------------------------------------------------

EOF
