#!/bin/bash


if [ ! -e $PWD/manage.py ]; then
    echo
    echo -e "\e[1;31mMake sure to execute this in the top directory.\e[0m"
    echo
    exit 1
fi
# ------------------------------------------
# Handling parameters
# ------------------------------------------
ARGS=$(getopt -o e:hs --long env:,help,skip -- "$@")
eval set -- "$ARGS"
while true; do
    case "$1" in
        -e | --env ) ENV=$2; shift 2 ;;
        -h | --help ) HELP=true; shift ;;
        -s | --skip ) SKIP=true; shift ;;
        -- ) shift; break ;;
        * ) break ;;
    esac
done
ENV=${ENV:-local}
HELP=${HELP:-false}
SKIP=${SKIP:-false}

declare -A ENVS
ENVS=(['test']=1 ['prod']=1 ['dev']=1 ['pilot']=1 ['local']=1)
if [[ ! ${ENVS[$ENV]} ]]; then
    echo
    echo -e "\e[1;31mSupplied env is not supported\e[0m"
    echo
    exit 1
fi

if $HELP; then
    cat << "EOF"
 This script updates your current activated env. Currently does not support Anaconda.

 The following is done in order:
  - Git pull --rebase the latest changes
  - Install requirements for specified env, default local
  - Execute migrations
  - Collect static if needed

 The following parameters are able to be used:
  -e | --env        Specify which environment should be installed, default local
  -h | --help       Raise this help
  -s | --skip       Skip the git pull --rebase
EOF
    exit 0
fi
# ------------------------------------------
# Detection of Python version
# ------------------------------------------
USE_PYTHON3=true
if ! which python3 >& /dev/null; then
    USE_PYTHON3=false
fi
if ! which python >& /dev/null && ! $USE_PYTHON3; then
    echo
    echo -e "\e[1;31mMake sure you have python3.4 or python3.5 installed and appended to your PATH.\e[0m"
    echo
    exit 1
fi

USE_PIP3=true
if ! which pip3 >& /dev/null; then
    USE_PIP3=false
fi
if ! which pip >& /dev/null && ! $USE_PIP3; then
    echo
    echo -e "\e[1;31mMake sure you have pip or pip3 installed and appended to your PATH.\e[0m"
    echo
    exit 1
fi

# ------------------------------------------
# Pulling changes
# ------------------------------------------
if ! $SKIP; then
    echo
    echo -e "\e[1;32mPulling changes\e[0m"
    echo
    git pull --rebase || exit 1
fi
# ------------------------------------------
# Installing requirements
# ------------------------------------------
echo
echo -e "\e[1;32mInstalling requirements\e[0m"
echo
if $USE_PIP3; then
    pip3 install --upgrade pip || exit 1
    pip3 install -r requirements/$ENV.txt || exit 1
else
    pip install --upgrade pip || exit 1
    pip install -r requirements/$ENV.txt || exit 1
fi

# ------------------------------------------
# Migrating
# ------------------------------------------
echo
echo -e "\e[1;32mMigrating\e[0m"
echo
if $USE_PYTHON3; then
    python3 manage.py migrate || exit 1
    python3 manage.py updatedata || exit 1
else
    python manage.py migrate || exit 1
    python manage.py updatedata || exit 1
fi

# ------------------------------------------
# Collecting static
# ------------------------------------------
if [ $ENV != "local" ]; then
    echo
    echo -e "\e[1;32mCollecting static\e[0m"
    echo
    if $USE_PYTHON3; then
        python3 manage.py collectstatic --no-input || exit 1
    else
        python manage.py collectstatic --no-input || exit 1
    fi
fi
# ------------------------------------------
# Compiling translations
# ------------------------------------------
echo
echo -e "\e[1;32mCompiling translations\e[0m"
echo
if $USE_PYTHON3; then
    python3 manage.py compilemessages -l 'en' -l 'nl' || exit 1
else
    python manage.py compilemessages -l 'en' -l 'nl' || exit 1
fi
# ------------------------------------------
# Finishing
# ------------------------------------------
echo
echo -e "\e[1;32mEnv updated!\e[0m"
echo
