import json

from rest_framework.test import APIClient
from rest_framework.test import APITestCase

from apps.MySU.models import AssociationMembership
from apps.MySU.models import AssociationSpecificData
from apps.MySU.models import AssociationSpecificDataFields
from apps.MySU.models import User
from tests.tests import fixtures


class AssociationMembershipTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username="s1234567")
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_delete_accepted(self):
        response = self.client.delete("/memberships/382d5570-5855-4bd7-90bd-17e575916cde")
        self.assertEqual(400, response.status_code, response.data)

    def test_delete_pending(self):
        self.user = User.objects.get(username="s1000000")
        self.client.force_authenticate(user=self.user)
        # Check if the membership that we want to delete exists
        response = self.client.get("/memberships/1fcd6c07-075f-4613-8bf8-16d4ec38fabf")
        self.assertEqual(200, response.status_code, response.data)
        # Check whether the related data field also exists
        response = self.client.get("/association_data/1fcd6c07-075f-4613-8bf8-16d4ec38fabf")
        self.assertEqual(200, response.status_code, response.data)
        # Do the actual delete
        response = self.client.delete("/memberships/1fcd6c07-075f-4613-8bf8-16d4ec38fabf")
        self.assertEqual(204, response.status_code, response.data)
        # Check that the membership is no more
        response = self.client.get("/memberships/1fcd6c07-075f-4613-8bf8-16d4ec38fabf")
        self.assertEqual(404, response.status_code, response.data)
        # and whether the related data field is no more
        response = self.client.get("/association_data/1fcd6c07-075f-4613-8bf8-16d4ec38fabf")
        self.assertEqual(404, response.status_code, response.data)

    def test_accept_membership(self):
        # This is a pending membership; get it to see that current=False
        response = self.client.get("/memberships/3ce35105-5042-4852-839f-3cc8af77f5ce")
        self.assertFalse(response.data["current"], response.data)
        # Accept the pending membership
        response = self.client.patch("/memberships/3ce35105-5042-4852-839f-3cc8af77f5ce", {"status": "Accepted"})
        self.assertEqual(200, response.status_code, response.data)
        self.assertEqual("Accepted", response.data["status"], response.data)
        # Check that current is properly updated
        self.assertTrue(response.data["current"], response.data)

    def test_reject_membership(self):
        # This is a pending membership; get it to see that current=False
        response = self.client.get("/memberships/3ce35105-5042-4852-839f-3cc8af77f5ce")
        self.assertFalse(response.data["current"], response.data)
        # Reject the pending membership
        response = self.client.patch("/memberships/3ce35105-5042-4852-839f-3cc8af77f5ce", {"status": "Rejected"})
        self.assertEqual(200, response.status_code, response.data)
        self.assertEqual("Rejected", response.data["status"], response.data)
        # Check that current is properly updated
        self.assertFalse(response.data["current"], response.data)

    def test_put_existing_profile(self):
        data = {
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "association": "/associations/student-union",
            "date_joined": "2019-10-06",
            "date_left": "",
            "status": ["Pending"],
            "type": "/membertypes/a9e124fa-51e8-410c-83d6-f6919907a839",
            "visible_after_date_left": "false"
        }

        response = self.client.put("/memberships", data=data)
        self.assertEqual(405, response.status_code, response.data)

    def test_patch(self):
        # Not editable fields
        # ----------------------------
        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6", data={"slug": "boe!"})
        self.assertEqual(200, response.status_code, msg=f"Tried to patch 'slug', {response.data}")
        self.assertEqual("ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6", response.data["slug"], msg="Slug has changed but shouldn't change.")

        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6",
                                     data={"assocation": "/associations/rising-sun-alienship-helios"})
        self.assertEqual(200, response.status_code, msg=f"Tried to patch 'association', {response.data}")
        self.assertEqual("http://testserver/associations/student-union", response.data['association'], msg="Association has changed but shouldn't change.")

        # Editable Fields
        # ----------------------------
        # valid input
        # ++++++++++++++
        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6",
                                     data={"date_joined": "2012-07-24"})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data="date_joined"))
        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6",
                                     data={"date_left": "2012-07-28"})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data="date_left"))
        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6",
                                     data={"visible_after_date_left": "True"})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}, {response}"
                         .format(data="visible_after_date_left", response=response.data))
        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6",
                                     data={"type": "/membertypes/a9e124fa-51e8-410c-83d6-f6919907a839"})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data="type"))
        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6",
                                     data={"status": "Accepted"})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data="status"))
        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6",
                                     data={"new_type": "/membertypes/a9e124fa-51e8-410c-83d6-f6919907a839"})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}".format(data="new_type"))
        # invalid input
        # ++++++++++++++
        response = self.client.patch("/memberships/ec47e37b-f30e-478f-9b7f-f9c7f3ca5aa6", data={"profile": "Boe!"})
        self.assertEqual(200, response.status_code, msg="Tried to patch {data}, {response}"
                         .format(data="profile", response=response.data))
        self.assertEqual("http://testserver/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c", response.data['profile'], msg="Patch of profile should have failed.")


    def test_post_invalid(self):
        data = {
            "profile": ""
        }
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def type_does_not_belong_to_assocation(self):
        data = {
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "association": "/associations/student-union",
            "date_joined": "2019-10-06",
            "date_left": "",
            "status": "Pending",
            "type": "/membertypes/9e3f9f5a-15f2-489f-bd94-a92fed0f4154",
            "visible_after_date_left": "false"
        }
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_duplicate(self):
        data = {
            "profile": "/profiles/be3f26fb-97d1-4270-8367-3e8539b3484d",
            "association": "/associations/student-union",
            "date_joined": "2019-10-06",
            "date_left": "",
            "status": "Pending",
            "type": "/membertypes/a9e124fa-51e8-410c-83d6-f6919907a839",
            "visible_after_date_left": "false"
        }
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_pending_no_data(self):
        data = {
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "association": "/associations/solar-barque-companionship-ra",
            "type": "/membertypes/c622b629-71b6-4b8e-a105-8ce99bb05104",
        }
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)

        data = {
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "association": "/associations/solar-barque-companionship-ra",
            "type": "/membertypes/c622b629-71b6-4b8e-a105-8ce99bb05104",
            "specific_data": []
        }
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_pending_minimal_data(self):
        data = {
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "association": "/associations/solar-barque-companionship-ra",
            "type": "/membertypes/c622b629-71b6-4b8e-a105-8ce99bb05104",
            "specific_data": [
                {
                    "data_field": "/association_data_fields/b25d534a-0799-4629-8d5a-d35bcb880ea5",
                    "value": "True"
                }
            ]
        }
        response = self.client.post("/memberships", data=data)
        self.assertEqual(201, response.status_code, response.data)
        specific_data_fields = AssociationSpecificDataFields.objects.filter(association__slug="solar-barque-companionship-ra")
        specific_data = AssociationSpecificData.objects.filter(membership__slug=response.data["slug"]).select_related("data_field")
        self.assertEqual(len(specific_data_fields), len(specific_data), specific_data)
        for data in specific_data:
            if data.data_field.slug == "b25d534a-0799-4629-8d5a-d35bcb880ea5":
                self.assertEqual("True", data.value, data.data_field.name)
            else:
                for field in specific_data_fields:
                    if data.data_field.slug == field.slug:
                        self.assertEqual(field.default, data.value, field.name)

    def test_post_pending_invoice(self):
        data = {
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "association": "/associations/solar-barque-companionship-ra",
            "date_joined": "2019-10-06",
            "date_left": None,
            "status": "Pending",
            "pay_by": AssociationMembership.PayBy.INVOICE.value,
            "type": "/membertypes/c622b629-71b6-4b8e-a105-8ce99bb05104",
            "visible_after_date_left": False,
            "specific_data": [
                {
                    "data_field": "/association_data_fields/b25d534a-0799-4629-8d5a-d35bcb880ea5",
                    "value": "False"
                }
            ]
        }
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)
        data["specific_data"][0]["value"] = "True"
        response = self.client.post("/memberships", data=data)
        self.assertEqual(201, response.status_code, response.data)

    def test_post_pending_debt_collection(self):
        data = {
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "association": "/associations/solar-barque-companionship-ra",
            "date_joined": "2019-10-06",
            "date_left": None,
            "status": "Pending",
            "pay_by": AssociationMembership.PayBy.DEBT_COLLECTION.value,
            "debt_collection_mandate_given": True,
            "type": "/membertypes/c622b629-71b6-4b8e-a105-8ce99bb05104",
            "visible_after_date_left": False,
            "specific_data": [
                {
                    "data_field": "/association_data_fields/b25d534a-0799-4629-8d5a-d35bcb880ea5",
                    "value": "False"
                }
            ]
        }
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)
        data["specific_data"][0]["value"] = "True"
        response = self.client.post("/memberships", data=data)
        self.assertEqual(201, response.status_code, response.data)

    def test_post_pending_notchosable(self):
        data = {
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "association": "/associations/solar-barque-companionship-ra",
            "date_joined": "2019-10-06",
            "date_left": "",
            "status": "Pending",
            "type": "/membertypes/6f9af56e-c6d3-4ff8-a49d-15501fdefb24",
            "visible_after_date_left": "false"
        }
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_post_status(self):
        data = {
            "profile": "/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c",
            "association": "/associations/solar-barque-companionship-ra",
            "date_joined": "2019-10-06",
            "date_left": None,
            "status": "Pending",
            "type": "/membertypes/c622b629-71b6-4b8e-a105-8ce99bb05104",
            "visible_after_date_left": "false",
            "specific_data": [
                {
                    "association": "/associations/solar-barque-companionship-ra",
                    "data_field": "/association_data_fields/b25d534a-0799-4629-8d5a-d35bcb880ea5",
                    "value": "True"
                }
            ],
        }
        response = self.client.post("/memberships", data=data, format="json")
        self.assertEqual(201, response.status_code, response.data)
        data["status"] = "Accepted"
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)
        data["status"] = "Claimed"
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)
        data["status"] = "Cancelled"
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)
        data["status"] = "Deleted"
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)
        data["status"] = "Disputed"
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)
        data["status"] = "Ended"
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)
        data["status"] = "Rejected"
        response = self.client.post("/memberships", data=data)
        self.assertEqual(400, response.status_code, response.data)

    def test_get(self):
        response = self.client.get("/memberships/382d5570-5855-4bd7-90bd-17e575916cde")
        self.assertEqual(200, response.status_code)

    def test_list(self):
        response = self.client.get("/memberships")
        self.assertEqual(200, response.status_code, response.data)
        self.assertEqual(response.data["count"], 7, msg="number of membertypes doesn't match expected number of membertypes")
        response = self.client.get("/memberships?expand=association_specific_data,profile,type,new_type&association__slug=student-union&limit = 10000")
        self.assertEqual(200, response.status_code, response.data)
        self.assertEqual(response.data["count"], 6, msg="number of membertypes doesn't match expected number of membertypes")


class AssociationMembershipResolveTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username="s7654321")
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def test_membership_resolve(self):
        data = {"38b0979c-ee9a-402b-aa90-f073d68d9386": "Accept"}
        response = self.client.post("/memberships/resolve", data=data, format="json")
        self.assertEqual(400, response.status_code, json.loads(response.content))
        self.assertEqual("MR2", json.loads(response.content)["code"], json.loads(response.content)["reason"])

        data = {"38b0979c-ee9a-402b-aa90-f073d68d9386": "accept"}
        response = self.client.post("/memberships/resolve", data=data, format="json")
        self.assertEqual(400, response.status_code, json.loads(response.content))
        self.assertEqual("MR3", json.loads(response.content)["code"], json.loads(response.content)["reason"])

        data = {"38b0979c-ee9a-402b-aa90-f073d68d9386": "accept", "9e8bc11b-2abf-4419-8483-fae11de9e543": "accept"}
        response = self.client.post("/memberships/resolve", data=data, format="json")
        self.assertEqual(400, response.status_code, json.loads(response.content))
        self.assertEqual("MR4", json.loads(response.content)["code"], json.loads(response.content)["reason"])

        data = {"38b0979c-ee9a-402b-aa90-f073d68d9386": "accept", "85f4dd97-3d8e-457c-8c87-47e4acac7e33": "deny"}
        response = self.client.post("/memberships/resolve", data=data, format="json")
        self.assertEqual(400, response.status_code, json.loads(response.content))
        self.assertEqual("MR2", json.loads(response.content)["code"], json.loads(response.content)["reason"])

        data = {"38b0979c-ee9a-402b-aa90-f073d68d9386": "accept", "85f4dd97-3d8e-457c-8c87-47e4acac7e33": "Disputed"}
        response = self.client.post("/memberships/resolve", data=data, format="json")
        self.assertEqual(200, response.status_code, json.loads(response.content))
