from rest_framework.test import APIClient
from rest_framework.test import APITestCase
from rest_framework_extensions import utils
from rest_framework_extensions.settings import extensions_api_settings

from apps.buddy.models import Matching
from apps.MySU.models import User
from tests.tests import fixtures


class MatchingTestCase(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.user = User.objects.get(username='s1234567')
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)
        self.protection_headers = {
            utils.prepare_header_name(extensions_api_settings.DEFAULT_BULK_OPERATION_HEADER_NAME): 'true'
        }

    def test_put_helios(self):
        data = {
            "url": "matchings/8a8f1509-071f-416b-a222-33208fb25059",
        }
        response = self.client.put('/matchings/8a8f1509-071f-416b-a222-33208fb25059', data)
        self.assertEqual(405, response.status_code, response.data)

    def test_put_student_union(self):
        data = {
            "url": "matchings/93f47e50-444e-450e-94b9-cabf535df342",
        }

        response = self.client.put('/matchings/93f47e50-444e-450e-94b9-cabf535df342', data)
        self.assertEqual(405, response.status_code, response.data)

    def test_patch_helios(self):
        data = {
            "url": "matchings/8a8f1509-071f-416b-a222-33208fb25059",
        }
        response = self.client.put('/matchings/8a8f1509-071f-416b-a222-33208fb25059', data)
        self.assertEqual(405, response.status_code, response.data)

    def test_patch_student_union(self):
        data = {
            "url": "matchings/93f47e50-444e-450e-94b9-cabf535df342",
        }

        response = self.client.put('/matchings/93f47e50-444e-450e-94b9-cabf535df342', data)
        self.assertEqual(405, response.status_code, response.data)

    def test_post_helios(self):
        data = {
            "association": "/associations/rising-sun-alienship-helios",
            "left_profile": '/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c',
            "right_profile": '/profiles/be3f26fb-97d1-4270-8367-3e8539b3484d'
        }
        response = self.client.post('/matchings', data=data)
        # Association field filter Helios from the selectable options. This means the POST request cannot find Helios.
        # As such the resulting status_code is 400 instead of 403.
        self.assertEqual(400, response.status_code, response.data)

    def test_post_student_union(self):
        data = {
            "association": "/associations/student-union",
            "left_profile": '/profiles/be3f26fb-97d1-4270-8367-3e8539b3484d',
            "right_profile": '/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c'
        }
        response = self.client.post('/matchings', data=data)
        self.assertEqual(201, response.status_code, response.data)

    def test_get_helios(self):
        # get a matching user is in
        response = self.client.get("/matchings/8a8f1509-071f-416b-a222-33208fb25059")
        self.assertEqual(200, response.status_code, "Error: {}, Input: Matching with the user for helios".format(response.data))
        # get a matching user is not in
        response = self.client.get("/matchings/4b621c7c-b826-4607-b860-2308266d149b")
        self.assertEqual(404, response.status_code, "Error: {}, Input: Matching without the user for helios".format(response.data))

    def test_get_student_union(self):
        # get a matching user is in
        response = self.client.get("/matchings/1ff8fcce-1c14-4e92-8eb3-dab69b51315b")
        self.assertEqual(200, response.status_code, "Error: {}, Input: Matching with the user for student union".format(response.data))
        # get a matching user is not in
        response = self.client.get("/matchings/90269bdd-3d6b-4405-bbe2-6284dcc43eca")
        self.assertEqual(200, response.status_code, "Error: {}, Input: Matching without the user for student union".format(response.data))

    def test_list(self):
        response = self.client.get('/matchings')
        self.assertEqual(200, response.status_code, "Error: {}, Input: List of all matchings".format(response.data))
        self.assertEqual(response.data['count'], 5, msg="number of matchings doesn't match expected number of matchings")

    def test_delete_helios(self):
        response = self.client.delete("/matchings/8a8f1509-071f-416b-a222-33208fb25059")
        self.assertEqual(403, response.status_code, "Error: {}, Input: Matching with the user for helios".format(response.data))
        # get a matching user is not in
        response = self.client.delete("/matchings/4b621c7c-b826-4607-b860-2308266d149b")
        self.assertEqual(404, response.status_code, "Error: {}, Input: Matching without the user for helios".format(response.data))

    def test_delete_student_union(self):
        # get a matching user is in
        response = self.client.delete("/matchings/1ff8fcce-1c14-4e92-8eb3-dab69b51315b")
        self.assertEqual(204, response.status_code, "Error: {}, Input: Matching with the user for student union".format(response.data))
        # get a matching user is not in
        response = self.client.delete("/matchings/90269bdd-3d6b-4405-bbe2-6284dcc43eca")
        self.assertEqual(204, response.status_code, "Error: {}, Input: Matching without the user for student union".format(response.data))

    def test_bulk_create(self):
        matching_1 = {
            "association": "/associations/student-union",
            "left_profile": '/profiles/be3f26fb-97d1-4270-8367-3e8539b3484d',
            "right_profile": '/profiles/75b5beed-97c2-4b43-bf3f-6c7ba85b3a5c'
        }
        matching_2 = {
            "association": "/associations/student-union",
            "left_profile": '/profiles/be3f26fb-97d1-4270-8367-3e8539b3484d',
            "right_profile": '/profiles/83d82e6d-9e9c-4705-a803-49865f32eab7'
        }
        data = [matching_1, matching_2]
        response = self.client.post('/matchings', data=data, format="json")
        self.assertEqual(201, response.status_code, response.data)

    def test_bulk_delete__without_header(self):
        number_of_matchings_SU_before = Matching.objects.filter(association__slug="student-union").count()
        number_of_matchings_H_before = Matching.objects.filter(association__slug="rising-sun-alienship-helios").count()

        matchings = [matching.slug for matching in Matching.objects.filter(association__slug="student-union").all()[0:2]]
        response = self.client.delete('/matchings?slug__in={}'.format(",".join(matchings)))
        self.assertEqual(400, response.status_code, response.data)

        number_of_matchings_SU_after = Matching.objects.filter(association__slug="student-union").count()
        number_of_matchings_H_after = Matching.objects.filter(association__slug="rising-sun-alienship-helios").count()
        self.assertEqual(number_of_matchings_SU_before, number_of_matchings_SU_after, "Number of matchings of student-union does not match")
        self.assertEqual(number_of_matchings_H_before, number_of_matchings_H_after, "Number of matchings of Helios does not match")

    def test_bulk_delete__with_header(self):
        number_of_matchings_SU_before = Matching.objects.filter(association__slug="student-union").count()
        number_of_matchings_H_before = Matching.objects.filter(association__slug="rising-sun-alienship-helios").count()

        matchings = [matching.slug for matching in Matching.objects.filter(association__slug="student-union").all()[0:2]]
        response = self.client.delete('/matchings?slug__in={}'.format(",".join(matchings)), **self.protection_headers)
        self.assertEqual(204, response.status_code, response.data)

        number_of_matchings_SU_after = Matching.objects.filter(association__slug="student-union").count()
        number_of_matchings_H_after = Matching.objects.filter(association__slug="rising-sun-alienship-helios").count()
        self.assertEqual(number_of_matchings_SU_before - 2, number_of_matchings_SU_after, "Number of matchings of student-union does not match")
        self.assertEqual(number_of_matchings_H_before, number_of_matchings_H_after, "Number of matchings of Helios does not match")

    def test_email_student_union(self):
        number_of_matchings_needing_emails = Matching.objects.filter(association__slug="student-union", email_send=False).count()
        self.assertEqual(3, number_of_matchings_needing_emails, "Number of matchings expected does not match the number in the fixtures")

        data = {
            "association": "/associations/student-union",
            "sent_to_all": False
        }
        response = self.client.post("/matchings/email", data=data, format="json")
        self.assertEqual(200, response.status_code, response.data)
        self.assertEqual(3, response.data["emails send"], response.data)

        number_of_matchings_needing_emails = Matching.objects.filter(association__slug="student-union", email_send=False).count()
        self.assertEqual(0, number_of_matchings_needing_emails, "Number of matchings expected the expected remaining number of matchings needing an email")

    def test_email_helios(self):
        number_of_matchings_needing_emails = Matching.objects.filter(association__slug="rising-sun-alienship-helios", email_send=False).count()
        self.assertEqual(2, number_of_matchings_needing_emails, "Number of matchings expected does not match the number in the fixtures")

        data = {
            "association": "/associations/helios",
            "sent_to_all": False
        }
        response = self.client.post("/matchings/email", data=data, format="json")
        self.assertEqual(400, response.status_code, response.data)

        number_of_matchings_needing_emails = Matching.objects.filter(association__slug="helios", email_send=False).count()
        self.assertEqual(0, number_of_matchings_needing_emails, "Number of matchings expected the expected remaining number of matchings needing an email")
