import json
from glob import glob

from django.conf import settings
from rest_framework.test import APIClient
from rest_framework.test import APITestCase

url_list = ['users', 'profiles', 'institutions', 'studies', 'associations', 'matchings',
            'memberships', 'membertypes', 'groups', 'group_memberships', 'events', 'event_types',
            'enrollments', 'enrollment_options', 'news', 'sepa_mandates']

url_list_inaccessible = ['permissions', 'global_permissions', 'object_permissions']


FIXTURE_DIRS = settings.FIXTURE_DIRS

url2model = {
    'users': 'user',
    'profiles': 'profile',
    'institutions': 'institution',
    'studies': 'studyrecord',
    'associations': 'association',
    'matchings': 'matching',
    'memberships': 'associationmembership',
    'membertypes': 'membertype',
    'groups': 'group',
    'group_memberships': 'groupmembership',
    'events': 'event',
    'event_types': 'eventtype',
    'enrollments': 'enrollment',
    'enrollment_options': 'enrollmentoption',
    'news': 'news',
    'sepa_mandates': 'sepamandate'
}


def fixture_crawler(keyword=None, recursive=False):
    fixture_dirs = []
    for _dir in FIXTURE_DIRS:
        if keyword is not None:
            fixture_dirs.extend(glob(_dir + '/' + keyword + '/*.json', recursive=recursive))
        else:
            fixture_dirs.extend(glob(_dir + '/*.json', recursive=recursive))
    return fixture_dirs


def create_test_statuscode_list(url, statuscode):
    def func1(self):
        resp = self.client.get('/' + url)
        self.assertEqual(resp.status_code, statuscode)

    func1.__name__ = 'test_{}_list'.format(url)
    return func1


def create_test_statuscode_detail(url, statuscode, slug):
    def func1(self):
        resp = self.client.get('/{}/{}'.format(url, slug))
        self.assertEqual(resp.status_code, statuscode)

    func1.__name__ = 'test_{}_detail'.format(url)
    return func1


def dict_generator(filenames):
    slugdict = {}
    for filename in filenames:
        with open(filename) as json_data:
            fixtures = json.load(json_data)
            for fixture in fixtures:
                model = fixture['model'].split('.')[1]
                try:
                    if model not in slugdict:
                        slugdict[model] = fixture['fields']['slug']
                except KeyError:
                    print("No slug field for this model {}".format(model))
    return slugdict


def generate_status_code_tests(status, urls, slug_dict):
    def decorator(class_):
        for url in urls:
            func = create_test_statuscode_list(url, status)
            if not hasattr(class_, func.__name__):
                setattr(class_, func.__name__, func)
            if url in url_list_inaccessible:
                func2 = create_test_statuscode_detail(url, status, [1])
                setattr(class_, func2.__name__, func2)
            else:
                model = url2model[url]
                try:
                    func2 = create_test_statuscode_detail(url, status, slug_dict[model])
                    if not hasattr(class_, func2.__name__):
                        setattr(class_, func2.__name__, func2)
                except KeyError:
                    print("No instances for model {}".format(model))
        return class_

    return decorator


fixtures = fixture_crawler(keyword='preset')
slug_dict = dict_generator(fixtures)


@generate_status_code_tests(401, url_list + url_list_inaccessible, slug_dict)
class AnonymousUrlTest(APITestCase):
    fixtures = fixtures

    def setUp(self):
        self.client = APIClient()

    def test_root(self):
        resp = self.client.get('/')
        self.assertEqual(resp.status_code, 401)

    def test_user_me(self):
        response = self.client.get('/users/me')
        self.assertEqual(401, response.status_code)

    def test_random_url(self):
        response = self.client.get('/adsf1234')
        self.assertEqual(404, response.status_code)
